#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import json
from unittest import mock

import sushy
from sushy import exceptions
from sushy.resources import constants as res_cons
from sushy.resources.eventservice import eventservice
from sushy.tests.unit import base


class EventServiceTestCase(base.TestCase):

    def setUp(self):
        super(EventServiceTestCase, self).setUp()
        self.conn = mock.Mock()
        with open('sushy/tests/unit/json_samples/eventservice.json') as f:
            self.json_doc = json.load(f)

        self.conn.get.return_value.json.return_value = self.json_doc

        self.eventservice = eventservice.EventService(
            self.conn, '/redfish/v1/EventService',
            redfish_version='1.0.8')

    def test__parse_attributes(self):
        self.eventservice._parse_attributes(self.json_doc)

        self.assertEqual(self.eventservice.identity, 'EventService')
        self.assertEqual(self.eventservice.name, 'Event Service')
        self.assertEqual(self.eventservice.delivery_retry_attempts, 3)
        self.assertEqual(self.eventservice.delivery_retry_interval, 30)
        self.assertEqual(self.eventservice.service_enabled, True)
        self.assertEqual(self.eventservice.status.health, res_cons.Health.OK)
        self.assertEqual(self.eventservice.status.health_rollup,
                         res_cons.Health.OK)
        self.assertEqual(self.eventservice.status.state,
                         res_cons.State.ENABLED)
        self.assertEqual(self.eventservice.subscriptions._path,
                         '/redfish/v1/EventService/Subscriptions/')

    def test__get_event_types_for_subscription(self):
        expected = set([sushy.EventType.STATUS_CHANGE,
                        sushy.EventType.RESOURCE_ADDED,
                        sushy.EventType.RESOURCE_REMOVED,
                        sushy.EventType.RESOURCE_UPDATED,
                        sushy.EventType.ALERT])

        values = self.eventservice.get_event_types_for_subscription()
        self.assertEqual(expected, values)
        self.assertIsInstance(values, set)

    def test__no_event_types_for_subscription(self):
        expected = set([sushy.EventType.STATUS_CHANGE,
                        sushy.EventType.RESOURCE_ADDED,
                        sushy.EventType.RESOURCE_REMOVED,
                        sushy.EventType.RESOURCE_UPDATED,
                        sushy.EventType.ALERT,
                        sushy.EventType.METRIC_REPORT,
                        sushy.EventType.OTHER])
        self.eventservice.event_types_for_subscription = []

        values = self.eventservice.get_event_types_for_subscription()
        self.assertEqual(expected, values)
        self.assertIsInstance(values, set)

    def test__eventservice_missing_action(self):
        self.eventservice._actions.submit_test_event = None
        self.assertRaisesRegex(
            exceptions.MissingActionError, '#EventService.SubmitTestEvent',
            self.eventservice.submit_test_event, mock.ANY, mock.ANY, mock.ANY,
            mock.ANY, mock.ANY, mock.ANY, mock.ANY, mock.ANY)

    def test__missing_submit_test_event_target(self):
        evt_json = self.eventservice.json
        evt_json['Actions']['#EventService.SubmitTestEvent'].pop('target')
        self.assertRaisesRegex(
            exceptions.MissingAttributeError,
            'attribute Actions/#EventService.SubmitTestEvent/target',
            self.eventservice._parse_attributes, self.json_doc)

    def test__submit_test_event(self):
        self.assertIsNotNone(self.eventservice._actions.submit_test_event)
        payload = {
            "EventId": "myEventId",
            "EventTimestamp": "2016-10-11T09:42:59Z",
            "EventType": "StatusChange",
            "Message": "This is a test message",
            "MessageArgs": ["arg0", "arg1"],
            "MessageId": "iLOEvents.0.9.ResourceStatusChanged",
            "OriginOfCondition": "/rest/v1/Chassis/1/FooBar",
            "Severity": "OK"
        }
        self.eventservice.submit_test_event(
            payload["EventId"], payload["EventTimestamp"],
            payload["EventType"], payload["Message"],
            payload["MessageArgs"], payload["MessageId"],
            payload["OriginOfCondition"], payload["Severity"])
        self.eventservice._conn.post.assert_called_once_with(
            '/redfish/v1/EventService/Actions/EventService.SubmitTestEvent/',
            data=payload)



# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


import json
from unittest import mock

from sushy import exceptions
from sushy.resources import base as sushy_base
from sushy.resources import constants as res_cons
from sushy.resources.registry import attribute_registry
from sushy.resources.registry import constants as reg_cons
from sushy.resources.registry import message_registry
from sushy.tests.unit import base


class MessageRegistryTestCase(base.TestCase):

    def setUp(self):
        super(MessageRegistryTestCase, self).setUp()
        self.conn = mock.Mock()
        with open('sushy/tests/unit/json_samples/message_registry.json') as f:
            self.json_doc = json.load(f)

        self.conn.get.return_value.json.return_value = self.json_doc

        self.registry = message_registry.MessageRegistry(
            self.conn, '/redfish/v1/Registries/Test',
            redfish_version='1.0.2')

    def test__parse_attributes(self):
        self.registry._parse_attributes(self.json_doc)
        self.assertEqual('Test.1.1.1', self.registry.identity)
        self.assertEqual('Test Message Registry', self.registry.name)
        self.assertEqual('en', self.registry.language)
        self.assertEqual('This registry defines messages for sushy testing',
                         self.registry.description)
        self.assertEqual('Test', self.registry.registry_prefix)
        self.assertEqual('1.1.1', self.registry.registry_version)
        self.assertEqual('sushy', self.registry.owning_entity)
        self.assertEqual(4, len(self.registry.messages))
        self.assertEqual('Everything OK',
                         self.registry.messages['Success'].description)
        self.assertEqual('Everything done successfully.',
                         self.registry.messages['Success'].message)
        self.assertEqual(res_cons.Severity.OK,
                         self.registry.messages['Success'].severity)
        self.assertEqual(0, self.registry.messages['Success'].number_of_args)
        self.assertEqual(2, len(self.registry.messages['TooBig'].param_types))
        self.assertEqual(reg_cons.MessageParamType.STRING,
                         self.registry.messages['TooBig'].param_types[0])
        self.assertEqual(reg_cons.MessageParamType.NUMBER,
                         self.registry.messages['TooBig'].param_types[1])
        self.assertEqual('Panic', self.registry.messages['Failed'].resolution)
        self.assertEqual(
            2, len(self.registry.messages['MissingThings'].param_types))
        self.assertEqual(res_cons.Severity.WARNING,
                         self.registry.messages['MissingThings'].severity)
        self.assertEqual(
            res_cons.PARAMTYPE_STRING,
            self.registry.messages['MissingThings'].param_types[0])
        self.assertEqual(
            res_cons.PARAMTYPE_NUMBER,
            self.registry.messages['MissingThings'].param_types[1])
        self.assertEqual(
            'Try Later', self.registry.messages['MissingThings'].resolution)

    def test__parse_attributes_return(self):
        attributes = self.registry._parse_attributes(self.json_doc)

        self.assertEqual({'Failed':
                          {'description': 'Nothing is OK',
                           'message': 'The property %1 broke everything.',
                           'number_of_args': 1,
                           'param_types': [reg_cons.MessageParamType.STRING],
                           'resolution': 'Panic',
                           'severity': res_cons.Severity.CRITICAL},
                          'MissingThings':
                          {'description': '',
                           'message':
                           "Property's %1 value cannot be less than %2.",
                           'number_of_args': 2,
                           'param_types': [reg_cons.MessageParamType.STRING,
                                           reg_cons.MessageParamType.NUMBER],
                           'resolution': 'Try Later',
                           'severity': res_cons.Severity.WARNING},
                          'Success':
                          {'description': 'Everything OK',
                           'message': 'Everything done successfully.',
                           'number_of_args': 0, 'param_types': None,
                           'resolution': 'None',
                           'severity': res_cons.Severity.OK},
                          'TooBig':
                          {'description': 'Value too big',
                           'message':
                           "Property's %1 value cannot be greater than %2.",
                           'number_of_args': 2,
                           'param_types': [reg_cons.MessageParamType.STRING,
                                           reg_cons.MessageParamType.NUMBER],
                           'resolution': 'Try again',
                           'severity': res_cons.Severity.WARNING}},
                         attributes.get('messages'))

    def test__parse_attributes_missing_msg_desc(self):
        self.json_doc['Messages']['Success'].pop('Description')
        self.registry._parse_attributes(self.json_doc)
        self.assertEqual('', self.registry.messages['Success'].description)

    def test__parse_attributes_missing_msg_severity(self):
        self.json_doc['Messages']['Success'].pop('Severity')
        self.registry._parse_attributes(self.json_doc)
        self.assertEqual(res_cons.Severity.WARNING,
                         self.registry.messages['Success'].severity)

    def test__parse_attributes_unknown_param_type(self):
        self.registry.json['Messages']['Failed']['ParamTypes'] = \
            ['unknown_type']
        self.assertRaisesRegex(exceptions.MalformedAttributeError,
                               'unknown_type',
                               self.registry._parse_attributes, self.json_doc)

    def test_parse_message(self):
        conn = mock.Mock()
        with open('sushy/tests/unit/json_samples/message_registry.json') as f:
            conn.get.return_value.json.return_value = json.load(f)
        registry = message_registry.MessageRegistry(
            conn, '/redfish/v1/Registries/Test',
            redfish_version='1.0.2')
        registries = {'Test.1.0.0': registry}
        message_field = sushy_base.MessageListField('Foo')
        message_field.message_id = 'Test.1.0.0.TooBig'
        message_field.message_args = ['arg1', 10]
        message_field.severity = None
        message_field.resolution = None

        parsed_msg = message_registry.parse_message(registries, message_field)

        self.assertEqual('Try again', parsed_msg.resolution)
        self.assertEqual(res_cons.Severity.WARNING, parsed_msg.severity)
        self.assertEqual('Property\'s arg1 value cannot be greater than 10.',
                         parsed_msg.message)

    def test_parse_message_with_severity_resolution_no_args(self):
        conn = mock.Mock()
        with open('sushy/tests/unit/json_samples/message_registry.json') as f:
            conn.get.return_value.json.return_value = json.load(f)
        registry = message_registry.MessageRegistry(
            conn, '/redfish/v1/Registries/Test',
            redfish_version='1.0.2')
        registries = {'Test.1.0.0': registry}
        message_field = sushy_base.MessageListField('Foo')
        message_field.message_id = 'Test.1.0.0.Success'
        message_field.severity = res_cons.Severity.OK
        message_field.resolution = 'Do nothing'

        parsed_msg = message_registry.parse_message(registries, message_field)

        self.assertEqual('Do nothing', parsed_msg.resolution)
        self.assertEqual(res_cons.Severity.OK, parsed_msg.severity)
        self.assertEqual('Everything done successfully.',
                         parsed_msg.message)

    def test_parse_message_bad_registry(self):
        conn = mock.Mock()
        with open('sushy/tests/unit/json_samples/message_registry.json') as f:
            conn.get.return_value.json.return_value = json.load(f)
        registry = message_registry.MessageRegistry(
            conn, '/redfish/v1/Registries/Test',
            redfish_version='1.0.2')
        registries = {'Test.1.0.0': registry}
        message_field = sushy_base.MessageListField('Foo')
        message_field.message_id = 'BadRegistry.TooBig'

        parsed_msg = message_registry.parse_message(registries, message_field)

        self.assertEqual(message_field, parsed_msg)

    def test_parse_message_bad_message_key_existing_message(self):
        conn = mock.Mock()
        with open('sushy/tests/unit/json_samples/message_registry.json') as f:
            conn.get.return_value.json.return_value = json.load(f)
        registry = message_registry.MessageRegistry(
            conn, '/redfish/v1/Registries/Test',
            redfish_version='1.0.2')
        registries = {'Test.1.0.0': registry}
        message_field = sushy_base.MessageListField('Foo')
        message_field.message_id = 'Test.1.0.0.BadMessageKey'
        message_field.message = 'Message'

        parsed_msg = message_registry.parse_message(registries, message_field)

        self.assertEqual(message_field.message, 'Message')
        self.assertEqual(message_field.message, parsed_msg.message)

    def test_parse_message_bad_message_key_no_existing_message(self):
        conn = mock.Mock()
        with open('sushy/tests/unit/json_samples/message_registry.json') as f:
            conn.get.return_value.json.return_value = json.load(f)
        registry = message_registry.MessageRegistry(
            conn, '/redfish/v1/Registries/Test',
            redfish_version='1.0.2')
        registries = {'Test.1.0.0': registry}
        message_field = sushy_base.MessageListField('Foo')
        message_field.message_id = 'Test.1.0.0.BadMessageKey'
        message_field.message = None

        parsed_msg = message_registry.parse_message(registries, message_field)

        self.assertEqual(message_field.message, 'unknown')
        self.assertEqual(message_field.message, parsed_msg.message)

    def test_parse_message_fallback_to_messages(self):
        conn = mock.Mock()
        with open('sushy/tests/unit/json_samples/message_registry.json') as f:
            conn.get.return_value.json.return_value = json.load(f)
        registry = message_registry.MessageRegistry(
            conn, '/redfish/v1/Registries/Test',
            redfish_version='1.0.2')
        registries = {'Messages': registry}
        message_field = sushy_base.MessageListField('Foo')
        message_field.message_id = 'Success'
        message_field.severity = res_cons.Severity.OK
        message_field.resolution = 'Do nothing'

        parsed_msg = message_registry.parse_message(registries, message_field)

        self.assertEqual('Do nothing', parsed_msg.resolution)
        self.assertEqual(res_cons.Severity.OK, parsed_msg.severity)
        self.assertEqual('Everything done successfully.',
                         parsed_msg.message)

    def test_parse_message_fallback_to_basemessages(self):
        conn = mock.Mock()
        with open('sushy/tests/unit/json_samples/message_registry.json') as f:
            conn.get.return_value.json.return_value = json.load(f)
        registry = message_registry.MessageRegistry(
            conn, '/redfish/v1/Registries/Test',
            redfish_version='1.0.2')
        registries = {'BaseMessages': registry}
        message_field = sushy_base.MessageListField('Foo')
        message_field.message_id = 'Success'
        message_field.severity = res_cons.Severity.OK
        message_field.resolution = 'Do nothing'

        parsed_msg = message_registry.parse_message(registries, message_field)

        self.assertEqual('Do nothing', parsed_msg.resolution)
        self.assertEqual(res_cons.Severity.OK, parsed_msg.severity)
        self.assertEqual('Everything done successfully.',
                         parsed_msg.message)

    def test_parse_message_fallback_failed(self):
        conn = mock.Mock()
        with open('sushy/tests/unit/json_samples/message_registry.json') as f:
            conn.get.return_value.json.return_value = json.load(f)
        registry = message_registry.MessageRegistry(
            conn, '/redfish/v1/Registries/Test',
            redfish_version='1.0.2')
        registries = {'Test.1.0.0': registry}
        message_field = sushy_base.MessageListField('Foo')
        message_field.message_id = 'BadMessageKey'
        message_field.message = None

        parsed_msg = message_registry.parse_message(registries, message_field)

        self.assertEqual(message_field.message, 'unknown')
        self.assertEqual(message_field.message, parsed_msg.message)

    def test_parse_message_not_enough_args(self):
        conn = mock.Mock()
        with open('sushy/tests/unit/json_samples/message_registry.json') as f:
            conn.get.return_value.json.return_value = json.load(f)
        registry = message_registry.MessageRegistry(
            conn, '/redfish/v1/Registries/Test',
            redfish_version='1.0.2')
        registries = {'Test.1.0.0': registry}
        message_field = sushy_base.MessageListField('Foo')
        message_field.message_id = 'Test.1.0.0.TooBig'
        message_field.message_args = ['arg1']
        message_field.severity = None
        message_field.resolution = None

        parsed_msg = message_registry.parse_message(registries, message_field)

        self.assertEqual('Try again', parsed_msg.resolution)
        self.assertEqual(res_cons.Severity.WARNING, parsed_msg.severity)
        self.assertEqual('Property\'s arg1 value cannot be greater than '
                         'unknown.', parsed_msg.message)

    def test_parse_message_multiple_registries(self):
        conn = mock.Mock()
        with open('sushy/tests/unit/json_samples/message_registry.json') as f:
            conn.get.return_value.json.return_value = json.load(f)
        msg_registry = message_registry.MessageRegistry(
            conn, '/redfish/v1/Registries/Test',
            redfish_version='1.0.2')
        attr_registry = attribute_registry.AttributeRegistry(
            self.conn, '/redfish/v1/Test/Bios/BiosRegistry',
            redfish_version='1.0.2')
        registries = {'Test.1.0.2': attr_registry,
                      'Test.1.0.0': msg_registry}
        message_field = sushy_base.MessageListField('Foo')
        message_field.message_id = 'Test.1.0.0.TooBig'
        message_field.message_args = ['arg1', 10]
        message_field.severity = None
        message_field.resolution = None

        parsed_msg = message_registry.parse_message(registries, message_field)

        self.assertEqual('Try again', parsed_msg.resolution)
        self.assertEqual(res_cons.Severity.WARNING, parsed_msg.severity)
        self.assertEqual('Property\'s arg1 value cannot be greater than 10.',
                         parsed_msg.message)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

# Values comes from the Redfish Chassis json-schema:
# https://redfish.dmtf.org/schemas/v1/Thermal.v1_7_1.json

import enum


class FanReadingUnit(enum.Enum):

    RPM = 'RPM'
    """The fan reading and thresholds are measured in revolutions per
    minute."""

    PERCENT = 'Percent'
    """The fan reading and thresholds are measured as a percentage."""


# Backward compatibility
FAN_READING_UNIT_PERCENTAGE = FanReadingUnit.PERCENT
FAN_READING_UNIT_RPM = FanReadingUnit.RPM



# -*- coding: utf-8 -*-

# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import json
from unittest import mock

from sushy.resources.chassis.thermal import constants as the_cons
from sushy.resources.chassis.thermal import thermal
from sushy.resources import constants as res_cons
from sushy.tests.unit import base


class ThermalTestCase(base.TestCase):

    def setUp(self):
        super(ThermalTestCase, self).setUp()
        self.conn = mock.Mock()
        with open('sushy/tests/unit/json_samples/thermal.json') as f:
            self.json_doc = json.load(f)

        self.conn.get.return_value.json.return_value = self.json_doc

        self.thermal = thermal.Thermal(
            self.conn, '/redfish/v1/Chassis/Blade1/Thermal',
            redfish_version='1.5.0')

    def test__parse_attributes(self):
        self.thermal._parse_attributes(self.json_doc)
        self.assertEqual('1.5.0', self.thermal.redfish_version)
        self.assertEqual('Thermal', self.thermal.identity)
        self.assertEqual('Blade Thermal', self.thermal.name)

        self.assertEqual('0', self.thermal.fans[0].identity)
        self.assertEqual('CPU Fan', self.thermal.fans[0].name)
        self.assertEqual('CPU', self.thermal.fans[0].physical_context)
        self.assertEqual(res_cons.State.ENABLED,
                         self.thermal.fans[0].status.state)
        self.assertEqual(res_cons.Health.OK,
                         self.thermal.fans[0].status.health)
        self.assertEqual(6000, self.thermal.fans[0].reading)
        self.assertEqual(the_cons.FanReadingUnit.RPM,
                         self.thermal.fans[0].reading_units)
        self.assertEqual(2000, self.thermal.fans[0].lower_threshold_fatal)
        self.assertEqual(0, self.thermal.fans[0].min_reading_range)
        self.assertEqual(10000, self.thermal.fans[0].max_reading_range)

        self.assertEqual('0', self.thermal.temperatures[0].identity)
        self.assertEqual('CPU Temp', self.thermal.temperatures[0].name)
        self.assertEqual(res_cons.State.ENABLED,
                         self.thermal.temperatures[0].status.state)
        self.assertEqual(res_cons.Health.OK,
                         self.thermal.temperatures[0].status.health)
        self.assertEqual(62, self.thermal.temperatures[0].reading_celsius)
        self.assertEqual(
            75,
            self.thermal.temperatures[0].upper_threshold_non_critical
        )
        self.assertEqual(
            90,
            self.thermal.temperatures[0].upper_threshold_critical
        )
        self.assertEqual(
            95,
            self.thermal.temperatures[0].upper_threshold_fatal
        )
        self.assertEqual(0,
                         self.thermal.temperatures[0].min_reading_range_temp)
        self.assertEqual(120,
                         self.thermal.temperatures[0].max_reading_range_temp)
        self.assertEqual('CPU', self.thermal.temperatures[0].physical_context)

    def test__parse_attributes_return(self):
        attributes = self.thermal._parse_attributes(self.json_doc)

        # Test that various types are returned correctly
        self.assertEqual([{'identity': '0',
                           'indicator_led': None,
                           'lower_threshold_critical': None,
                           'lower_threshold_fatal': 2000,
                           'lower_threshold_non_critical': None,
                           'manufacturer': None,
                           'max_reading_range': 10000,
                           'min_reading_range': 0,
                           'model': None,
                           'name': 'CPU Fan',
                           'part_number': None,
                           'physical_context': 'CPU',
                           'reading': 6000,
                           'reading_units': the_cons.FanReadingUnit.RPM,
                           'serial_number': None,
                           'status':
                           {'health': res_cons.Health.OK,
                            'health_rollup': None,
                            'state': res_cons.State.ENABLED},
                           'upper_threshold_critical': None,
                           'upper_threshold_fatal': None,
                           'upper_threshold_non_critical': None}],
                         attributes.get('fans'))
        self.assertEqual([{'identity': '0',
                           'lower_threshold_critical': None,
                           'lower_threshold_fatal': None,
                           'lower_threshold_non_critical': None,
                           'max_allowable_operating_value': None,
                           'max_reading_range_temp': 120,
                           'min_allowable_operating_value': None,
                           'min_reading_range_temp': 0,
                           'name': 'CPU Temp',
                           'physical_context': 'CPU',
                           'reading_celsius': 62,
                           'sensor_number': None,
                           'status': {'health': res_cons.Health.OK,
                                      'health_rollup': None,
                                      'state': res_cons.State.ENABLED},
                           'upper_threshold_critical': 90,
                           'upper_threshold_fatal': 95,
                           'upper_threshold_non_critical': 75}],
                         attributes.get('temperatures'))



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import json
from unittest import mock


from sushy.resources.oem import fake
from sushy.resources.system import system
from sushy.tests.unit import base


class FakeOEMSystemExtensionTestCase(base.TestCase):

    def setUp(self):
        super(FakeOEMSystemExtensionTestCase, self).setUp()
        self.conn = mock.MagicMock()
        with open('sushy/tests/unit/json_samples/system.json', 'r') as f:
            self.conn.get.return_value.json.return_value = json.loads(f.read())

        self.sys_instance = system.System(
            self.conn, '/redfish/v1/Systems/437XR1138R2',
            redfish_version='1.0.2')
        self.fake_sys_oem_extn = fake.FakeOEMSystemExtension(
            self.conn, '',
            redfish_version='1.0.2')
        self.fake_sys_oem_extn = self.fake_sys_oem_extn.set_parent_resource(
            self.sys_instance, 'Contoso')

    def test__parse_oem_attributes(self):
        self.assertEqual('Contoso OEM system', self.fake_sys_oem_extn.name)
        self.assertEqual('#Contoso.ComputerSystem',
                         self.fake_sys_oem_extn.data_type)
        self.assertEqual('PacWest Production Facility', (
            self.fake_sys_oem_extn.production_location.facility_name))
        self.assertEqual('USA', (
            self.fake_sys_oem_extn.production_location.country))
        self.assertEqual(
            "/redfish/v1/Systems/437XR1138R2/Oem/Contoso/Actions/"
            "Contoso.Reset",
            self.fake_sys_oem_extn._actions.reset.target_uri)

    def test_get_reset_system_path(self):
        value = self.fake_sys_oem_extn.get_reset_system_path()
        expected = (
            '/redfish/v1/Systems/437XR1138R2/Oem/Contoso/Actions/Contoso.Reset'
            )
        self.assertEqual(expected, value)



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import json
from unittest import mock

from sushy import exceptions
from sushy.resources import constants as res_cons
from sushy.resources.updateservice import constants as ups_cons
from sushy.resources.updateservice import softwareinventory
from sushy.resources.updateservice import updateservice
from sushy import taskmonitor
from sushy.tests.unit import base


class UpdateServiceTestCase(base.TestCase):

    def setUp(self):
        super(UpdateServiceTestCase, self).setUp()
        self.conn = mock.Mock()
        with open('sushy/tests/unit/json_samples/updateservice.json') as f:
            self.json_doc = json.load(f)

        self.conn.get.return_value.json.return_value = self.json_doc

        self.upd_serv = updateservice.UpdateService(
            self.conn, '/redfish/v1/UpdateService/UpdateService',
            redfish_version='1.3.0')

    def test__parse_attributes(self):
        self.upd_serv._parse_attributes(self.json_doc)
        self.assertEqual('UpdateService', self.upd_serv.identity)
        self.assertEqual('/FWUpdate', self.upd_serv.http_push_uri)
        self.assertIn('/FWUpdate', self.upd_serv.http_push_uri_targets)
        self.assertFalse(self.upd_serv.http_push_uri_targets_busy)
        self.assertEqual('Update service', self.upd_serv.name)
        self.assertTrue(self.upd_serv.service_enabled)
        self.assertEqual(res_cons.State.ENABLED, self.upd_serv.status.state)
        self.assertEqual(res_cons.Health.OK, self.upd_serv.status.health)
        self.assertEqual(
            res_cons.Health.OK,
            self.upd_serv.status.health_rollup)

    def test__parse_attributes_missing_actions(self):
        self.upd_serv.json.pop('Actions')
        self.assertRaisesRegex(
            exceptions.MissingAttributeError, 'attribute Actions',
            self.upd_serv._parse_attributes, self.json_doc)

    def test_simple_update(self):
        with open('sushy/tests/unit/json_samples/task_monitor.json') as f:
            task_json = json.load(f)
        task_submitted = mock.Mock()
        task_submitted.json.return_value = task_json
        task_submitted.status_code = 202
        task_submitted.headers = {'Content-Length': 42,
                                  'Location': '/Task/545'}
        self.conn.post.return_value = task_submitted

        tm = self.upd_serv.simple_update(
            image_uri='local.server/update.exe',
            targets=['/redfish/v1/UpdateService/FirmwareInventory/BMC'],
            transfer_protocol=ups_cons.UpdateTransferProtocolType.HTTPS)

        self.assertIsInstance(tm, taskmonitor.TaskMonitor)
        self.assertEqual('/Task/545', tm.task_monitor_uri)

        self.upd_serv._conn.post.assert_called_once_with(
            '/redfish/v1/UpdateService/Actions/SimpleUpdate',
            data={
                'ImageURI': 'local.server/update.exe',
                'Targets': ['/redfish/v1/UpdateService/FirmwareInventory/BMC'],
                'TransferProtocol': 'HTTPS'})

    def test_simple_update_task_uri(self):
        with open('sushy/tests/unit/json_samples/task.json') as f:
            task_json = json.load(f)
        task_submitted = mock.Mock()
        task_submitted.json.return_value = task_json
        task_submitted.status_code = 202
        task_submitted.headers = {'Content-Length': 42,
                                  'Location': '/Taskmonitor/545'}
        self.conn.post.return_value = task_submitted

        tm = self.upd_serv.simple_update(
            image_uri='local.server/update.exe',
            targets=['/redfish/v1/UpdateService/FirmwareInventory/BMC'],
            transfer_protocol=ups_cons.UpdateTransferProtocolType.HTTPS)

        self.assertIsInstance(tm, taskmonitor.TaskMonitor)
        self.assertEqual('/redfish/v1/TaskService/Tasks/545',
                         tm.task_monitor_uri)

        self.upd_serv._conn.post.assert_called_once_with(
            '/redfish/v1/UpdateService/Actions/SimpleUpdate',
            data={
                'ImageURI': 'local.server/update.exe',
                'Targets': ['/redfish/v1/UpdateService/FirmwareInventory/BMC'],
                'TransferProtocol': 'HTTPS'})

    def test_simple_update_missing_location(self):
        with open('sushy/tests/unit/json_samples/task_monitor.json') as f:
            task_json = json.load(f)
        task_submitted = mock.Mock()
        task_submitted.json.return_value = task_json
        task_submitted.status_code = 202
        task_submitted.headers = {'Allow': 'GET'}
        self.conn.post.return_value = task_submitted

        self.assertRaises(
            exceptions.MissingHeaderError,
            self.upd_serv.simple_update,
            image_uri='local.server/update.exe',
            targets='/redfish/v1/UpdateService/Actions/SimpleUpdate',
            transfer_protocol='HTTPS')

    def test_simple_update_backward_compatible_protocol(self):
        with open('sushy/tests/unit/json_samples/task.json') as f:
            task_json = json.load(f)
        task_submitted = mock.Mock()
        task_submitted.json.return_value = task_json
        task_submitted.status_code = 202
        task_submitted.headers = {'Content-Length': 42,
                                  'Location': '/Task/545'}
        self.conn.post.return_value = task_submitted

        self.upd_serv.simple_update(
            image_uri='local.server/update.exe',
            targets='/redfish/v1/UpdateService/Actions/SimpleUpdate',
            transfer_protocol='HTTPS')
        self.upd_serv._conn.post.assert_called_once_with(
            '/redfish/v1/UpdateService/Actions/SimpleUpdate',
            data={
                'ImageURI': 'local.server/update.exe',
                'Targets': '/redfish/v1/UpdateService/Actions/SimpleUpdate',
                'TransferProtocol': 'HTTPS'})

    def test_simple_update_without_target(self):
        with open('sushy/tests/unit/json_samples/task.json') as f:
            task_json = json.load(f)
        task_submitted = mock.Mock()
        task_submitted.json.return_value = task_json
        task_submitted.status_code = 202
        task_submitted.headers = {'Content-Length': 42,
                                  'Location': '/Task/545'}
        self.conn.post.return_value = task_submitted
        self.upd_serv.simple_update(
            image_uri='local.server/update.exe',
            transfer_protocol='HTTPS')
        self.upd_serv._conn.post.assert_called_once_with(
            '/redfish/v1/UpdateService/Actions/SimpleUpdate',
            data={
                'ImageURI': 'local.server/update.exe',
                'TransferProtocol': 'HTTPS'})

    def test_simple_update_bad_protocol(self):
        self.assertRaises(
            exceptions.InvalidParameterValueError,
            self.upd_serv.simple_update,
            image_uri='local.server/update.exe',
            targets='/redfish/v1/UpdateService/Actions/SimpleUpdate',
            transfer_protocol='ROYAL')

    @mock.patch.object(softwareinventory, 'SoftwareInventoryCollection',
                       autospec=True)
    def test_software_inventory(self, software_inventory_collection_mock):
        self.upd_serv.software_inventory
        software_inventory_collection_mock.assert_called_once_with(
            self.conn, '/redfish/v1/UpdateService/SoftwareInventory',
            self.upd_serv.redfish_version,
            self.upd_serv._registries,
            self.upd_serv.root)

    @mock.patch.object(softwareinventory, 'SoftwareInventoryCollection',
                       autospec=True)
    def test_firmware_inventory(self, software_inventory_collection_mock):
        self.upd_serv.firmware_inventory
        software_inventory_collection_mock.assert_called_once_with(
            self.conn, '/redfish/v1/UpdateService/FirmwareInventory',
            self.upd_serv.redfish_version,
            self.upd_serv._registries, self.upd_serv.root)


class UpdateServiceNoInvTestCase(base.TestCase):

    def setUp(self):
        super(UpdateServiceNoInvTestCase, self).setUp()
        self.conn = mock.Mock()
        no_inv_json = 'sushy/tests/unit/json_samples/updateservice_no_inv.json'
        with open(no_inv_json) as f:
            self.json_doc = json.load(f)

        self.conn.get.return_value.json.return_value = self.json_doc

        self.upd_serv = updateservice.UpdateService(
            self.conn, '/redfish/v1/UpdateService/UpdateService',
            redfish_version='1.3.0')

    def test_software_inventory_when_sw_inv_absent(self):
        self.assertRaisesRegex(
            exceptions.MissingAttributeError,
            'SoftwareInventory/@odata.id',
            getattr, self.upd_serv, 'software_inventory')

    def test_firmware_inventory_when_fw_inv_absent(self):
        self.assertRaisesRegex(
            exceptions.MissingAttributeError,
            'FirmwareInventory/@odata.id',
            getattr, self.upd_serv, 'firmware_inventory')



#    Copyright (c) 2021 Anexia Internetdienstleistungs GmbH
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import json
from unittest import mock

from sushy.resources import constants as res_cons
from sushy.resources.system.network import adapter
from sushy.resources.system.network import device_function
from sushy.resources.system.network import port
from sushy.tests.unit import base


class NetworkAdapterTestCase(base.TestCase):

    def setUp(self):
        super(NetworkAdapterTestCase, self).setUp()
        self.conn = mock.Mock()
        with open('sushy/tests/unit/json_samples/network_adapter.json') as f:
            self.json_doc = json.load(f)

        self.conn.get.return_value.json.return_value = self.json_doc

        self.adapter = adapter.NetworkAdapter(
            self.conn, '/redfish/v1/Chassis/System.Embedded.1/NetworkAdapters/'
                       'NIC.Integrated.1',
            redfish_version='1.0.2')

    def test__parse_attributes(self):
        self.adapter._parse_attributes(self.json_doc)
        self.assertEqual('1.0.2', self.adapter.redfish_version)
        self.assertEqual('NIC.Integrated.1', self.adapter.identity)
        self.assertEqual('Network Adapter View', self.adapter.name)
        self.assertEqual('Network Adapter View', self.adapter.description)
        self.assertEqual('0R887V', self.adapter.part_number)
        self.assertEqual('IL7403101N00ZJ', self.adapter.serial_number)
        self.assertEqual('Mellanox Technologies', self.adapter.manufacturer)
        self.assertEqual('MLNX 25GbE 2P ConnectX4LX RNDC', self.adapter.model)
        self.assertEqual(res_cons.STATE_ENABLED, self.adapter.status.state)
        self.assertEqual(res_cons.HEALTH_OK, self.adapter.status.health)
        self.assertEqual(res_cons.HEALTH_OK, self.adapter.status.health_rollup)

    def test_network_ports(self):
        self.conn.get.return_value.json.reset_mock()
        with open('sushy/tests/unit/'
                  'json_samples/network_port_collection.json') as f:
            self.conn.get.return_value.json.return_value = json.load(f)
        actual_network_ports = self.adapter.network_ports
        self.assertIsInstance(actual_network_ports,
                              port.NetworkPortCollection)
        self.conn.get.return_value.json.assert_called_once_with()

    def test_network_ports_cached(self):
        self.conn.get.return_value.json.reset_mock()
        with open('sushy/tests/unit/'
                  'json_samples/network_port_collection.json') as f:
            self.conn.get.return_value.json.return_value = json.load(f)
        actual_ports = self.adapter.network_ports
        self.conn.get.return_value.json.reset_mock()
        self.assertIs(actual_ports,
                      self.adapter.network_ports)
        self.conn.get.return_value.json.assert_not_called()

    def test_network_ports_refresh(self):
        with open('sushy/tests/unit/'
                  'json_samples/network_port_collection.json') as f:
            self.conn.get.return_value.json.return_value = json.load(f)
        ports = self.adapter.network_ports
        self.assertIsInstance(ports, port.NetworkPortCollection)

        with open('sushy/tests/unit/'
                  'json_samples/network_adapter.json') as f:
            self.conn.get.return_value.json.return_value = json.load(f)

        self.adapter.invalidate()
        self.adapter.refresh(force=False)

        self.assertTrue(ports._is_stale)

        with open('sushy/tests/unit/'
                  'json_samples/network_port_collection.json') as f:
            self.conn.get.return_value.json.return_value = json.load(f)
        self.assertIsInstance(self.adapter.network_ports,
                              port.NetworkPortCollection)

    def test_network_device_functions(self):
        self.conn.get.return_value.json.reset_mock()
        with open('sushy/tests/unit/json_samples/'
                  'network_device_function_collection.json') as f:
            self.conn.get.return_value.json.return_value = json.load(f)
        actual_functions = self.adapter.network_device_functions
        self.assertIsInstance(actual_functions,
                              device_function.NetworkDeviceFunctionCollection)
        self.conn.get.return_value.json.assert_called_once_with()

    def test_network_device_functions_cached(self):
        self.conn.get.return_value.json.reset_mock()
        with open('sushy/tests/unit/json_samples/'
                  'network_device_function_collection.json') as f:
            self.conn.get.return_value.json.return_value = json.load(f)
        actual_functions = self.adapter.network_device_functions
        self.conn.get.return_value.json.reset_mock()
        self.assertIs(actual_functions,
                      self.adapter.network_device_functions)
        self.conn.get.return_value.json.assert_not_called()

    def test_network_device_functions_refresh(self):
        with open('sushy/tests/unit/json_samples/'
                  'network_device_function_collection.json') as f:
            self.conn.get.return_value.json.return_value = json.load(f)
        functions = self.adapter.network_device_functions
        self.assertIsInstance(functions,
                              device_function.NetworkDeviceFunctionCollection)

        with open('sushy/tests/unit/json_samples/'
                  'network_adapter.json') as f:
            self.conn.get.return_value.json.return_value = json.load(f)

        self.adapter.invalidate()
        self.adapter.refresh(force=False)

        self.assertTrue(functions._is_stale)

        with open('sushy/tests/unit/json_samples/'
                  'network_device_function_collection.json') as f:
            self.conn.get.return_value.json.return_value = json.load(f)
        self.assertIsInstance(self.adapter.network_device_functions,
                              device_function.NetworkDeviceFunctionCollection)


class NetworkAdapterCollectionTestCase(base.TestCase):

    def setUp(self):
        super(NetworkAdapterCollectionTestCase, self).setUp()
        self.conn = mock.Mock()
        with open('sushy/tests/unit/json_samples/'
                  'network_adapter_collection.json') as f:
            self.json_doc = json.load(f)

        self.conn.get.return_value.json.return_value = self.json_doc

        self.adapter_col = adapter.NetworkAdapterCollection(
            self.conn, '/redfish/v1/Chassis/System.Embedded.1/NetworkAdapters',
            redfish_version='1.0.2')

    def test__parse_attributes(self):
        self.adapter_col._parse_attributes(self.json_doc)
        self.assertEqual('1.0.2', self.adapter_col.redfish_version)
        self.assertEqual('Network Adapter Collection', self.adapter_col.name)
        self.assertEqual(('/redfish/v1/Chassis/System.Embedded.1/'
                          'NetworkAdapters/NIC.Integrated.1',),
                         self.adapter_col.members_identities)

    @mock.patch.object(adapter, 'NetworkAdapter', autospec=True)
    def test_get_member(self, mock_interface):
        self.adapter_col.get_member(
            '/redfish/v1/Chassis/System.Embedded.1/'
            'NetworkAdapters/NIC.Integrated.1'
        )
        mock_interface.assert_called_once_with(
            self.adapter_col._conn,
            '/redfish/v1/Chassis/System.Embedded.1/'
            'NetworkAdapters/NIC.Integrated.1',
            redfish_version=self.adapter_col.redfish_version,
            registries=None, root=self.adapter_col.root)

    @mock.patch.object(adapter, 'NetworkAdapter', autospec=True)
    def test_get_members(self, mock_interface):
        members = self.adapter_col.get_members()
        calls = [
            mock.call(self.adapter_col._conn,
                      '/redfish/v1/Chassis/System.Embedded.1/'
                      'NetworkAdapters/NIC.Integrated.1',
                      redfish_version=self.adapter_col.redfish_version,
                      registries=None, root=self.adapter_col.root),
        ]
        mock_interface.assert_has_calls(calls)
        self.assertIsInstance(members, list)
        self.assertEqual(1, len(members))



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# This is referred from Redfish standard schema.
# https://redfish.dmtf.org/schemas/CompositionService.v1_1_0.json

import logging

from sushy import exceptions
from sushy.resources import base
from sushy.resources import common
from sushy.resources.compositionservice import resourceblock
from sushy.resources.compositionservice import resourcezone
from sushy import utils

LOG = logging.getLogger(__name__)


class CompositionService(base.ResourceBase):

    allow_overprovisioning = base.Field('AllowOverprovisioning')
    """This indicates whether this service is allowed to overprovision"""

    allow_zone_affinity = base.Field('AllowZoneAffinity')
    """This indicates whether a client is allowed to request that given
    composition request"""

    description = base.Field('Description')
    """The composition service description"""

    identity = base.Field('Id', required=True)
    """The composition service identity string"""

    name = base.Field('Name', required=True)
    """The composition service name"""

    status = common.StatusField('Status')
    """The status of composition service"""

    service_enabled = base.Field('ServiceEnabled')
    """The status of composition service is enabled"""

    def __init__(self, connector, identity, redfish_version=None,
                 registries=None, root=None):
        """A class representing a CompositionService

        :param connector: A connector instance
        :param identity: The identity of the CompositionService resource
        :param redfish_version: The version of RedFish. Used to construct
            the object according to schema of given version
        :param registries: Dict of Redfish Message Registry objects to be
            used in any resource that needs registries to parse messages
        :param root: Sushy root object. Empty for Sushy root itself.
        """
        super(CompositionService, self).__init__(
            connector, identity, redfish_version=redfish_version,
            registries=registries, root=root)

    def _get_resource_blocks_collection_path(self):
        """Helper function to find the ResourceBlockCollections path"""
        res_block_col = self.json.get('ResourceBlocks')
        if not res_block_col:
            raise exceptions.MissingAttributeError(
                attribute='ResourceBlocks', resource=self._path)
        return res_block_col.get('@odata.id')

    def _get_resource_zones_collection_path(self):
        """Helper function to find the ResourceZoneCollections path"""
        res_zone_col = self.json.get('ResourceZones')
        if not res_zone_col:
            raise exceptions.MissingAttributeError(
                attribute='ResourceZones', resource=self._path)
        return res_zone_col.get('@odata.id')

    @property
    @utils.cache_it
    def resource_blocks(self):
        """Property to reference `ResourceBlockCollection` instance"""
        return resourceblock.ResourceBlockCollection(
            self._conn, self._get_resource_blocks_collection_path,
            redfish_version=self.redfish_version, registries=self.registries,
            root=self.root)

    @property
    @utils.cache_it
    def resource_zones(self):
        """Property to reference `ResourceZoneCollection` instance"""
        return resourcezone.ResourceZoneCollection(
            self._conn, self._get_resource_zones_collection_path,
            redfish_version=self.redfish_version, registries=self.registries,
            root=self.root)



# Copyright 2017 Red Hat, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

# Values come from the Redfish System json-schema 1.0.0:
# http://redfish.dmtf.org/schemas/v1/ComputerSystem.v1_0_0.json#/definitions/ComputerSystem  # noqa

import enum

from sushy.resources import constants as res_cons

# Reset action constants

RESET_ON = res_cons.ResetType.ON
RESET_FORCE_OFF = res_cons.ResetType.FORCE_OFF
RESET_GRACEFUL_SHUTDOWN = res_cons.ResetType.GRACEFUL_SHUTDOWN
RESET_GRACEFUL_RESTART = res_cons.ResetType.GRACEFUL_RESTART
RESET_FORCE_RESTART = res_cons.ResetType.FORCE_RESTART
RESET_NMI = res_cons.ResetType.NMI
RESET_FORCE_ON = res_cons.ResetType.FORCE_ON
RESET_PUSH_POWER_BUTTON = res_cons.ResetType.PUSH_POWER_BUTTON

# System' PowerState constants

SYSTEM_POWER_STATE_ON = res_cons.POWER_STATE_ON
"""The system is powered on"""

SYSTEM_POWER_STATE_OFF = res_cons.POWER_STATE_OFF
"""The system is powered off, although some components may continue to
   have AUX power such as management controller"""

SYSTEM_POWER_STATE_POWERING_ON = res_cons.POWER_STATE_POWERING_ON
"""A temporary state between Off and On. This temporary state can
   be very short"""

SYSTEM_POWER_STATE_POWERING_OFF = res_cons.POWER_STATE_POWERING_OFF
"""A temporary state between On and Off. The power off action can take
   time while the OS is in the shutdown process"""


class BootSource(enum.Enum):
    """Boot source target constants"""

    NONE = 'None'
    """Boot from the normal boot device."""

    PXE = 'Pxe'
    """Boot from the Pre-Boot EXecution (PXE) environment."""

    FLOPPY = 'Floppy'
    """Boot from the floppy disk drive."""

    CD = 'Cd'
    """Boot from the CD or DVD."""

    USB = 'Usb'
    """Boot from a system BIOS-specified USB device."""

    HDD = 'Hdd'
    """Boot from a hard drive."""

    BIOS_SETUP = 'BiosSetup'
    """Boot to the BIOS setup utility."""

    UTILITIES = 'Utilities'
    """Boot to the manufacturer's utilities program or programs."""

    DIAGS = 'Diags'
    """Boot to the manufacturer's diagnostics program."""

    UEFI_SHELL = 'UefiShell'
    """Boot to the UEFI Shell."""

    UEFI_TARGET = 'UefiTarget'
    """Boot to the UEFI device specified in the UefiTargetBootSourceOverride
    property."""

    SD_CARD = 'SDCard'
    """Boot from an SD card."""

    UEFI_HTTP = 'UefiHttp'
    """Boot from a UEFI HTTP network location."""

    REMOTE_DRIVE = 'RemoteDrive'
    """Boot from a remote drive, such as an iSCSI target."""

    UEFI_BOOT_NEXT = 'UefiBootNext'
    """Boot to the UEFI device that the BootNext property specifies."""

    USB_CD = 'UsbCd'
    """Boot from a USB CD device as specified by the system BIOS.

    **This is NOT a standard value!**
    On SuperMicro X11 and X12 machines, virtual media is presented as an USB CD
    drive as opposed to a CD drive. Both are present in the list of boot
    devices, however only selecting UsbCd as the boot source results in a
    successful boot from vMedia. If CD is selected, boot fails even if vMedia
    is inserted."""


# Backward compatibility
BOOT_SOURCE_TARGET_NONE = BootSource.NONE
BOOT_SOURCE_TARGET_PXE = BootSource.PXE
BOOT_SOURCE_TARGET_FLOPPY = BootSource.FLOPPY
BOOT_SOURCE_TARGET_CD = BootSource.CD
BOOT_SOURCE_TARGET_USB = BootSource.USB
BOOT_SOURCE_TARGET_HDD = BootSource.HDD
BOOT_SOURCE_TARGET_BIOS_SETUP = BootSource.BIOS_SETUP
BOOT_SOURCE_TARGET_UTILITIES = BootSource.UTILITIES
BOOT_SOURCE_TARGET_DIAGS = BootSource.DIAGS
BOOT_SOURCE_TARGET_UEFI_SHELL = BootSource.UEFI_SHELL
BOOT_SOURCE_TARGET_UEFI_TARGET = BootSource.UEFI_TARGET
BOOT_SOURCE_TARGET_SD_CARD = BootSource.SD_CARD
BOOT_SOURCE_TARGET_UEFI_HTTP = BootSource.UEFI_HTTP
BOOT_SOURCE_TARGET_USB_CD = BootSource.USB_CD


class BootSourceOverrideMode(enum.Enum):
    """Boot source mode constants"""

    LEGACY = 'Legacy'
    """The system boots in non-UEFI boot mode to the boot source override
    target."""

    UEFI = 'UEFI'
    """The system boots in UEFI boot mode to the boot source override
    target."""


# Backward compatibility
BOOT_SOURCE_MODE_BIOS = BootSourceOverrideMode.LEGACY
BOOT_SOURCE_MODE_UEFI = BootSourceOverrideMode.UEFI


class BootSourceOverrideEnabled(enum.Enum):
    """Boot source enabled constants"""

    DISABLED = 'Disabled'
    """The system boots normally."""

    ONCE = 'Once'
    """On its next boot cycle, the system boots one time to the boot source
    override target.  Then, the BootSourceOverrideEnabled value is reset
    to `Disabled`."""

    CONTINUOUS = 'Continuous'
    """The system boots to the target specified in the
    BootSourceOverrideTarget property until this property is `Disabled`."""


# Backward compatibility
BOOT_SOURCE_ENABLED_DISABLED = BootSourceOverrideEnabled.DISABLED
BOOT_SOURCE_ENABLED_ONCE = BootSourceOverrideEnabled.ONCE
BOOT_SOURCE_ENABLED_CONTINUOUS = BootSourceOverrideEnabled.CONTINUOUS


class SystemType(enum.Enum):
    """System type constants"""

    PHYSICAL = 'Physical'
    """A computer system."""

    VIRTUAL = 'Virtual'
    """A virtual machine instance running on this system."""

    OS = 'OS'
    """An operating system instance."""

    PHYSICALLY_PARTITIONED = 'PhysicallyPartitioned'
    """A hardware-based partition of a computer system."""

    VIRTUALLY_PARTITIONED = 'VirtuallyPartitioned'
    """A virtual or software-based partition of a computer system."""

    COMPOSED = 'Composed'
    """A computer system constructed by binding resource blocks together."""

    DPU = 'DPU'
    """A computer system that performs the functions of a data processing
    unit, such as a SmartNIC."""


# Backward compatibility
SYSTEM_TYPE_PHYSICAL = SystemType.PHYSICAL
SYSTEM_TYPE_VIRTUAL = SystemType.VIRTUAL
SYSTEM_TYPE_OS = SystemType.OS
SYSTEM_TYPE_PHYSICALLY_PARTITIONED = SystemType.PHYSICALLY_PARTITIONED
SYSTEM_TYPE_VIRTUALLY_PARTITIONED = SystemType.VIRTUALLY_PARTITIONED
SYSTEM_TYPE_COMPOSED = SystemType.COMPOSED
SYSTEM_TYPE_DPU = SystemType.DPU


# Processor related constants
# Values comes from the Redfish Processor json-schema 1.3.0:
# http://redfish.dmtf.org/schemas/v1/Processor.v1_3_0.json

class ProcessorArchitecture(enum.Enum):
    """Processor Architecture constants"""

    X86 = 'x86'
    """x86 or x86-64."""

    IA_64 = 'IA-64'
    """Intel Itanium."""

    ARM = 'ARM'
    """ARM."""

    MIPS = 'MIPS'
    """MIPS."""

    POWER = 'Power'
    """Power."""

    OEM = 'OEM'
    """OEM-defined."""


# Backward compatibility
PROCESSOR_ARCH_x86 = ProcessorArchitecture.X86
PROCESSOR_ARCH_IA_64 = ProcessorArchitecture.IA_64
PROCESSOR_ARCH_ARM = ProcessorArchitecture.ARM
PROCESSOR_ARCH_MIPS = ProcessorArchitecture.MIPS
PROCESSOR_ARCH_OEM = ProcessorArchitecture.OEM


class ProcessorType(enum.Enum):
    """Processor type constants"""

    CPU = 'CPU'
    """A CPU."""

    GPU = 'GPU'
    """A GPU."""

    FPGA = 'FPGA'
    """An FPGA."""

    DSP = 'DSP'
    """A DSP."""

    ACCELERATOR = 'Accelerator'
    """An accelerator."""

    CORE = 'Core'
    """A core in a processor."""

    THREAD = 'Thread'
    """A thread in a processor."""

    OEM = 'OEM'
    """An OEM-defined processing unit."""


# Backward compatibility
PROCESSOR_TYPE_CPU = ProcessorType.CPU
PROCESSOR_TYPE_GPU = ProcessorType.GPU
PROCESSOR_TYPE_FPGA = ProcessorType.FPGA
PROCESSOR_TYPE_DSP = ProcessorType.DSP
PROCESSOR_TYPE_ACCELERATOR = ProcessorType.ACCELERATOR
PROCESSOR_TYPE_CORE = ProcessorType.CORE
PROCESSOR_TYPE_THREAD = ProcessorType.THREAD
PROCESSOR_TYPE_OEM = ProcessorType.OEM


class InstructionSet(enum.Enum):
    """Processor InstructionSet constants"""

    X86 = 'x86'
    """x86 32-bit."""

    X86_64 = 'x86-64'
    """x86 64-bit."""

    IA_64 = 'IA-64'
    """Intel IA-64."""

    ARM_A32 = 'ARM-A32'
    """ARM 32-bit."""

    ARM_A64 = 'ARM-A64'
    """ARM 64-bit."""

    MIPS32 = 'MIPS32'
    """MIPS 32-bit."""

    MIPS64 = 'MIPS64'
    """MIPS 64-bit."""

    POWER_ISA = 'PowerISA'
    """PowerISA-64 or PowerISA-32."""

    OEM = 'OEM'
    """OEM-defined."""


PROCESSOR_INSTRUCTIONSET_ARM_A32 = InstructionSet.ARM_A32
PROCESSOR_INSTRUCTIONSET_ARM_A64 = InstructionSet.ARM_A64
PROCESSOR_INSTRUCTIONSET_IA_64 = InstructionSet.IA_64
PROCESSOR_INSTRUCTIONSET_MIPS32 = InstructionSet.MIPS32
PROCESSOR_INSTRUCTIONSET_MIPS64 = InstructionSet.MIPS64
PROCESSOR_INSTRUCTIONSET_OEM = InstructionSet.OEM
PROCESSOR_INSTRUCTIONSET_x86 = InstructionSet.X86
PROCESSOR_INSTRUCTIONSET_x86_64 = InstructionSet.X86_64


# Secure boot constants from SecureBoot schema version 1.1.0
# https://redfish.dmtf.org/schemas/v1/SecureBoot.v1_1_0.json
# Some names were altered for clarity

class SecureBootCurrentBoot(enum.Enum):
    ENABLED = 'Enabled'
    """UEFI Secure Boot is currently enabled."""

    DISABLED = 'Disabled'
    """UEFI Secure Boot is currently disabled."""


# Backward compatibility
SECURE_BOOT_ENABLED = SecureBootCurrentBoot.ENABLED
SECURE_BOOT_DISABLED = SecureBootCurrentBoot.DISABLED


class SecureBootMode(enum.Enum):
    SETUP = 'SetupMode'
    """UEFI Secure Boot is currently in Setup Mode."""

    USER = 'UserMode'
    """UEFI Secure Boot is currently in User Mode."""

    AUDIT = 'AuditMode'
    """UEFI Secure Boot is currently in Audit Mode."""

    DEPLOYED = 'DeployedMode'
    """UEFI Secure Boot is currently in Deployed Mode."""


# Backward compatibility
SECURE_BOOT_MODE_SETUP = SecureBootMode.SETUP
SECURE_BOOT_MODE_USER = SecureBootMode.USER
SECURE_BOOT_MODE_AUDIT = SecureBootMode.AUDIT
SECURE_BOOT_MODE_DEPLOYED = SecureBootMode.DEPLOYED


class SecureBootResetKeysType(enum.Enum):
    RESET_ALL_KEYS_TO_DEFAULT = 'ResetAllKeysToDefault'
    """Reset the contents of all UEFI Secure Boot key databases, including
    the PK key database, to the default values."""

    DELETE_ALL_KEYS = 'DeleteAllKeys'
    """Delete the contents of all UEFI Secure Boot key databases, including
    the PK key database.  This puts the system in Setup Mode."""

    DELETE_PK = 'DeletePK'
    """Delete the contents of the PK UEFI Secure Boot database.  This puts
    the system in Setup Mode."""


# Internal constant based on
# https://redfish.dmtf.org/schemas/v1/SecureBootDatabase.v1_0_1.json
_SECURE_BOOT_DATABASE_RESET_KEYS = frozenset([
    SecureBootResetKeysType.RESET_ALL_KEYS_TO_DEFAULT,
    SecureBootResetKeysType.DELETE_ALL_KEYS,
])


# Backward compatibility
SECURE_BOOT_RESET_KEYS_TO_DEFAULT = \
    SecureBootResetKeysType.RESET_ALL_KEYS_TO_DEFAULT
SECURE_BOOT_RESET_KEYS_DELETE_ALL = SecureBootResetKeysType.DELETE_ALL_KEYS
SECURE_BOOT_RESET_KEYS_DELETE_PK = SecureBootResetKeysType.DELETE_PK


class SecureBootDatabaseId(enum.Enum):
    # This enumeration is hand-written based on the database schema
    # https://redfish.dmtf.org/schemas/v1/SecureBootDatabase.v1_0_1.json
    # and the UEFI specification.

    PLATFORM_KEY = "PK"
    KEY_EXCHANGE_KEYS = "KEK"
    ALLOWED_KEYS_DATABASE = "db"
    DENIED_KEYS_DATABASE = "dbx"
    RECOVERY_KEYS_DATABASE = "dbr"
    TIMESTAMP_DATABASE = "dbt"

    DEFAULT_PLATFORM_KEY = "PKDefault"
    DEFAULT_KEY_EXCHANGE_KEYS = "KEKDefault"
    DEFAULT_ALLOWED_KEYS_DATABASE = "dbDefault"
    DEFAULT_DENIED_KEYS_DATABASE = "dbxDefault"
    DEFAULT_RECOVERY_KEYS_DATABASE = "dbrDefault"
    DEFAULT_TIMESTAMP_DATABASE = "dbtDefault"


# Backward compatibility
SECURE_BOOT_PLATFORM_KEY = SecureBootDatabaseId.PLATFORM_KEY
SECURE_BOOT_KEY_EXCHANGE_KEYS = SecureBootDatabaseId.KEY_EXCHANGE_KEYS
SECURE_BOOT_ALLOWED_KEYS_DATABASE = SecureBootDatabaseId.ALLOWED_KEYS_DATABASE
SECURE_BOOT_DENIED_KEYS_DATABASE = SecureBootDatabaseId.DENIED_KEYS_DATABASE
SECURE_BOOT_RECOVERY_KEYS_DATABASE = \
    SecureBootDatabaseId.RECOVERY_KEYS_DATABASE
SECURE_BOOT_TIMESTAMP_DATABASE = SecureBootDatabaseId.TIMESTAMP_DATABASE

SECURE_BOOT_DEFAULT_PLATFORM_KEY = SecureBootDatabaseId.DEFAULT_PLATFORM_KEY
SECURE_BOOT_DEFAULT_KEY_EXCHANGE_KEYS = \
    SecureBootDatabaseId.DEFAULT_KEY_EXCHANGE_KEYS
SECURE_BOOT_DEFAULT_ALLOWED_KEYS_DATABASE = \
    SecureBootDatabaseId.DEFAULT_ALLOWED_KEYS_DATABASE
SECURE_BOOT_DEFAULT_DENIED_KEYS_DATABASE = \
    SecureBootDatabaseId.DEFAULT_DENIED_KEYS_DATABASE
SECURE_BOOT_DEFAULT_RECOVERY_KEYS_DATABASE = \
    SecureBootDatabaseId.DEFAULT_RECOVERY_KEYS_DATABASE
SECURE_BOOT_DEFAULT_TIMESTAMP_DATABASE = \
    SecureBootDatabaseId.DEFAULT_TIMESTAMP_DATABASE



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import json
from unittest import mock

from sushy.resources import constants as res_cons
from sushy.resources.system.network import constants as net_cons
from sushy.resources.system.network import port
from sushy.tests.unit import base


class NetworkPortTestCase(base.TestCase):

    def setUp(self):
        super(NetworkPortTestCase, self).setUp()
        self.conn = mock.Mock()
        with open('sushy/tests/unit/json_samples/network_port.json') as f:
            self.json_doc = json.load(f)

        self.conn.get.return_value.json.return_value = self.json_doc

        self.port = port.NetworkPort(
            self.conn, '/redfish/v1/Chassis/System.Embedded.1/NetworkAdapters/'
                       '/NIC.Integrated.1/NetworkPorts/NIC.Integrated.1-1',
            redfish_version='1.0.2')

    def test__parse_attributes(self):
        self.port._parse_attributes(self.json_doc)
        self.assertEqual('1.0.2', self.port.redfish_version)
        self.assertEqual('NIC.Integrated.1-1', self.port.identity)
        self.assertEqual('Network Port View', self.port.name)
        self.assertEqual('Network Port View', self.port.description)
        self.assertEqual(res_cons.State.ENABLED, self.port.status.state)
        self.assertEqual(res_cons.Health.OK, self.port.status.health)
        self.assertEqual(res_cons.Health.OK, self.port.status.health_rollup)
        self.assertEqual(['01:02:03:04:05:06'],
                         self.port.associated_network_addresses)
        self.assertEqual(10000, self.port.current_link_speed_mbps)
        self.assertEqual(net_cons.FlowControl.NONE,
                         self.port.flow_control_configuration)
        self.assertEqual(net_cons.FlowControl.NONE,
                         self.port.flow_control_status)
        self.assertEqual(net_cons.LinkStatus.UP, self.port.link_status)


class NetworkPortCollectionTestCase(base.TestCase):

    def setUp(self):
        super(NetworkPortCollectionTestCase, self).setUp()
        self.conn = mock.Mock()
        with open('sushy/tests/unit/json_samples/'
                  'network_port_collection.json') as f:
            self.json_doc = json.load(f)

        self.conn.get.return_value.json.return_value = self.json_doc

        self.port_col = port.NetworkPortCollection(
            self.conn, '/redfish/v1/Chassis/System.Embedded.1/NetworkAdapters/'
                       'NIC.Integrated.1/NetworkPorts',
            redfish_version='1.0.2')

    def test__parse_attributes(self):
        self.port_col._parse_attributes(self.json_doc)
        self.assertEqual((
            '/redfish/v1/Chassis/System.Embedded.1/NetworkAdapters/'
            'NIC.Integrated.1/NetworkPorts/NIC.Integrated.1-1',
            '/redfish/v1/Chassis/System.Embedded.1/NetworkAdapters/'
            'NIC.Integrated.1/NetworkPorts/NIC.Integrated.1-2'
        ),
            self.port_col.members_identities)

    @mock.patch.object(port, 'NetworkPort', autospec=True)
    def test_get_member(self, NetworkPort_mock):
        self.port_col.get_member(
            '/redfish/v1/Chassis/System.Embedded.1/NetworkAdapters/'
            'NIC.Integrated.1/NetworkPorts/NIC.Integrated.1-1'
        )
        NetworkPort_mock.assert_called_once_with(
            self.port_col._conn,
            '/redfish/v1/Chassis/System.Embedded.1/NetworkAdapters/'
            'NIC.Integrated.1/NetworkPorts/NIC.Integrated.1-1',
            redfish_version=self.port_col.redfish_version, registries=None,
            root=self.port_col.root)

    @mock.patch.object(port, 'NetworkPort', autospec=True)
    def test_get_members(self, NetworkPort_mock):
        members = self.port_col.get_members()
        NetworkPort_mock.assert_has_calls([
            mock.call(
                self.port_col._conn,
                '/redfish/v1/Chassis/System.Embedded.1/NetworkAdapters/'
                'NIC.Integrated.1/NetworkPorts/NIC.Integrated.1-1',
                redfish_version=self.port_col.redfish_version, registries=None,
                root=self.port_col.root
            ),
            mock.call(
                self.port_col._conn,
                '/redfish/v1/Chassis/System.Embedded.1/NetworkAdapters/'
                'NIC.Integrated.1/NetworkPorts/NIC.Integrated.1-2',
                redfish_version=self.port_col.redfish_version, registries=None,
                root=self.port_col.root
            ),
        ])
        self.assertIsInstance(members, list)
        self.assertEqual(2, len(members))



# Copyright 2017 Red Hat, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import json
from unittest import mock

from sushy.resources import constants as res_cons
from sushy.resources.registry import message_registry
from sushy.resources import settings
from sushy.tests.unit import base


class SettingsFieldTestCase(base.TestCase):

    def setUp(self):
        super(SettingsFieldTestCase, self).setUp()
        with open('sushy/tests/unit/json_samples/settings.json') as f:
            self.json = json.load(f)

        self.settings = settings.SettingsField()

        conn = mock.Mock()
        with open('sushy/tests/unit/json_samples/message_registry.json') as f:
            conn.get.return_value.json.return_value = json.load(f)
        registry = message_registry.MessageRegistry(
            conn, '/redfish/v1/Registries/Test',
            redfish_version='1.0.2')
        self.registries = {'Test.1.0': registry}

    @mock.patch.object(settings, 'LOG', autospec=True)
    def test__load(self, mock_LOG):
        instance = self.settings._load(self.json, mock.Mock())

        self.assertEqual('9234ac83b9700123cc32',
                         instance._etag)
        self.assertEqual('2016-03-07T14:44:30-05:00',
                         instance.time)
        self.assertEqual('/redfish/v1/Systems/437XR1138R2/BIOS/Settings',
                         instance._settings_object_idref.resource_uri)
        self.assertEqual('Test.1.0.Failed',
                         instance.messages[0].message_id)
        self.assertEqual('Settings %1 update failed due to invalid value',
                         instance.messages[0].message)
        self.assertEqual(res_cons.Severity.CRITICAL,
                         instance.messages[0].severity)
        self.assertEqual('Fix the value and try again',
                         instance.messages[0].resolution)
        self.assertEqual('arg1',
                         instance.messages[0].message_args[0])
        self.assertEqual('#/Attributes/ProcTurboMode',
                         instance.messages[0]._related_properties[0])
        self.assertEqual('/redfish/v1/Systems/437XR1138R2/BIOS/Settings',
                         instance._settings_object_idref.resource_uri)
        self.assertEqual([res_cons.ApplyTime.ON_RESET,
                          res_cons.ApplyTime.IN_MAINTENANCE_WINDOW_ON_RESET],
                         instance._supported_apply_times)
        self.assertIsNone(instance.maintenance_window)
        mock_LOG.warning.assert_called_once()
        mock_LOG.reset_mock()
        self.assertIsNone(instance.operation_apply_time_support)
        mock_LOG.warning.assert_called_once()

    def test_commit(self):
        conn = mock.Mock()
        instance = self.settings._load(self.json, conn)
        instance.commit(conn, {'Attributes': {'key': 'value'}})
        conn.patch.assert_called_once_with(
            '/redfish/v1/Systems/437XR1138R2/BIOS/Settings',
            data={'Attributes': {'key': 'value'}})

    def test_get_status_failure(self):
        instance = self.settings._load(self.json, mock.Mock())

        status = instance.get_status(self.registries)
        self.assertEqual(status.status,
                         settings.UPDATE_FAILURE)
        self.assertEqual(status.messages[0].severity,
                         res_cons.Severity.CRITICAL)
        self.assertEqual(status.messages[0].message,
                         'The property arg1 broke everything.')

    def test_get_status_success(self):
        instance = self.settings._load(self.json, mock.Mock())
        instance.messages[0].message_id = 'Test.1.0.Success'
        instance.messages[0].severity = res_cons.Severity.OK
        status = instance.get_status(self.registries)
        self.assertEqual(status.status,
                         settings.UPDATE_SUCCESS)
        self.assertEqual(status.messages[0].severity, res_cons.Severity.OK)
        self.assertEqual(status.messages[0].message,
                         'Everything done successfully.')

    def test_get_status_noupdates(self):
        instance = self.settings._load(self.json, mock.Mock())
        instance.time = None
        status = instance.get_status(self.registries)
        self.assertEqual(status.status,
                         settings.NO_UPDATES)
        self.assertIsNone(status.messages)



# Copyright 2017 Red Hat, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import datetime
import json
from unittest import mock

import sushy
from sushy import exceptions
from sushy.resources import base as resource_base
from sushy.resources.system import system
from sushy.tests.unit import base
from sushy import utils


class UtilsTestCase(base.TestCase):

    def test_revert_dictionary(self):
        source = {'key0': 'value0', 'key1': 'value1'}
        expected = {'value0': 'key0', 'value1': 'key1'}
        self.assertEqual(expected, utils.revert_dictionary(source))

    @mock.patch.object(utils.LOG, 'warning', autospec=True)
    def test_get_members_identities(self, log_mock):
        members = [{"@odata.id": "/redfish/v1/Systems/FOO"},
                   {"other_key": "/redfish/v1/Systems/FUN"},
                   {"@odata.id": "/redfish/v1/Systems/BAR/"}]
        expected = ('/redfish/v1/Systems/FOO', '/redfish/v1/Systems/BAR')
        self.assertEqual(expected, utils.get_members_identities(members))
        self.assertEqual(1, log_mock.call_count)

    def test_int_or_none(self):
        self.assertEqual(1, utils.int_or_none('1'))
        self.assertIsNone(None, utils.int_or_none(None))

    def test_bool_or_none_none(self):
        self.assertIsNone(utils.bool_or_none(None))

    def test_bool_or_none_bool(self):
        self.assertEqual(True, utils.bool_or_none(True))

    def setUp(self):
        super(UtilsTestCase, self).setUp()
        self.conn = mock.MagicMock()
        with open('sushy/tests/unit/json_samples/system.json') as f:
            system_json = json.load(f)
        self.conn.get.return_value.json.return_value = system_json

        self.sys_inst = system.System(self.conn,
                                      '/redfish/v1/Systems/437XR1138R2',
                                      redfish_version='1.0.2')

    def test_get_sub_resource_path_by(self):
        subresource_path = 'EthernetInterfaces'
        expected_result = '/redfish/v1/Systems/437XR1138R2/EthernetInterfaces'
        value = utils.get_sub_resource_path_by(self.sys_inst,
                                               subresource_path)
        self.assertEqual(expected_result, value)

    def test_get_sub_resource_path_by_list(self):
        subresource_path = ['EthernetInterfaces']
        expected_result = '/redfish/v1/Systems/437XR1138R2/EthernetInterfaces'
        value = utils.get_sub_resource_path_by(self.sys_inst,
                                               subresource_path)
        self.assertEqual(expected_result, value)

    def test_get_sub_resource_path_by_collection(self):
        subresource_path = ["Links", "ManagedBy"]
        expected_result = ['/redfish/v1/Managers/BMC']
        value = utils.get_sub_resource_path_by(self.sys_inst,
                                               subresource_path,
                                               is_collection=True)
        self.assertEqual(expected_result, value)

    def test_get_sub_resource_path_by_fails(self):
        subresource_path = ['Links', 'Chassis']
        expected_result = 'attribute Links/Chassis/@odata.id is missing'
        self.assertRaisesRegex(
            exceptions.MissingAttributeError,
            expected_result,
            utils.get_sub_resource_path_by,
            self.sys_inst, subresource_path)

    def test_get_sub_resource_path_by_fails_with_empty_path(self):
        self.assertRaisesRegex(
            ValueError,
            '"subresource_name" cannot be empty',
            utils.get_sub_resource_path_by,
            self.sys_inst, [])

    def test_get_sub_resource_path_by_fails_with_empty_string(self):
        self.assertRaisesRegex(
            ValueError,
            '"subresource_name" cannot be empty',
            utils.get_sub_resource_path_by,
            self.sys_inst, '')

    def test_max_safe(self):
        self.assertEqual(10, utils.max_safe([1, 3, 2, 8, 5, 10, 6]))
        self.assertEqual(821, utils.max_safe([15, 300, 270, None, 821, None]))
        self.assertEqual(0, utils.max_safe([]))
        self.assertIsNone(utils.max_safe([], default=None))

    def test_camelcase_to_underscore_joined(self):
        input_vs_expected = [
            ('GarbageCollection', 'garbage_collection'),
            ('DD', 'dd'),
            ('rr', 'rr'),
            ('AABbbC', 'aa_bbb_c'),
            ('AABbbCCCDd', 'aa_bbb_ccc_dd'),
            ('Manager', 'manager'),
            ('EthernetInterfaceCollection', 'ethernet_interface_collection'),
            (' ', ' '),
        ]
        for inp, exp in input_vs_expected:
            self.assertEqual(exp, utils.camelcase_to_underscore_joined(inp))

    def test_camelcase_to_underscore_joined_fails_with_empty_string(self):
        self.assertRaisesRegex(
            ValueError,
            '"camelcase_str" cannot be empty',
            utils.camelcase_to_underscore_joined, '')


class NestedResource(resource_base.ResourceBase):

    def _parse_attributes(self, json_doc):
        pass


class BaseResource(resource_base.ResourceBase):

    def _parse_attributes(self, json_doc):
        pass

    def _do_some_crunch_work_to_get_a(self):
        return 'a'

    @utils.cache_it
    def get_a(self):
        return self._do_some_crunch_work_to_get_a()

    def _do_some_crunch_work_to_get_b(self):
        return 'b'

    @utils.cache_it
    def get_b(self):
        return self._do_some_crunch_work_to_get_b()

    @property
    @utils.cache_it
    def nested_resource(self):
        return NestedResource(
            self._conn, "path/to/nested_resource",
            redfish_version=self.redfish_version)

    @property
    @utils.cache_it
    def few_nested_resources(self):
        return [NestedResource(self._conn, "/nested_res1",
                               redfish_version=self.redfish_version),
                NestedResource(self._conn, "/nested_res2",
                               redfish_version=self.redfish_version)]


class CacheTestCase(base.TestCase):

    def setUp(self):
        super(CacheTestCase, self).setUp()
        self.conn = mock.Mock()
        self.res = BaseResource(connector=self.conn, path='/Foo',
                                redfish_version='1.0.2')

    def test_cache_nested_resource_retrieval(self):
        nested_res = self.res.nested_resource
        few_nested_res = self.res.few_nested_resources

        self.assertIsInstance(nested_res, NestedResource)
        self.assertIs(nested_res, self.res.nested_resource)
        self.assertIsInstance(few_nested_res, list)
        for n_res in few_nested_res:
            self.assertIsInstance(n_res, NestedResource)
        self.assertIs(few_nested_res, self.res.few_nested_resources)

        self.res.invalidate()
        self.res.refresh(force=False)

        self.assertIsNotNone(self.res._cache_nested_resource)
        self.assertTrue(self.res._cache_nested_resource._is_stale)
        self.assertIsNotNone(self.res._cache_few_nested_resources)
        for n_res in self.res._cache_few_nested_resources:
            self.assertTrue(n_res._is_stale)

        self.assertIsInstance(self.res.nested_resource, NestedResource)
        self.assertFalse(self.res._cache_nested_resource._is_stale)
        self.assertIsInstance(self.res.few_nested_resources, list)
        for n_res in self.res._cache_few_nested_resources:
            self.assertFalse(n_res._is_stale)

    def test_cache_non_resource_retrieval(self):
        with mock.patch.object(
                self.res, '_do_some_crunch_work_to_get_a',
                wraps=self.res._do_some_crunch_work_to_get_a,
                autospec=True) as do_work_to_get_a_spy:
            result = self.res.get_a()
            self.assertTrue(do_work_to_get_a_spy.called)

            do_work_to_get_a_spy.reset_mock()
            # verify subsequent invocation
            self.assertEqual(result, self.res.get_a())
            self.assertFalse(do_work_to_get_a_spy.called)

    def test_cache_clear_only_selected_attr(self):
        self.res.nested_resource
        self.res.get_a()
        self.res.get_b()

        utils.cache_clear(self.res, False, only_these=['get_a'])

        # cache cleared (set to None)
        self.assertIsNone(self.res._cache_get_a)
        # cache retained
        self.assertEqual('b', self.res._cache_get_b)
        self.assertFalse(self.res._cache_nested_resource._is_stale)

    def test_cache_clear_failure(self):
        self.assertRaises(
            TypeError, utils.cache_clear, self.res, False, only_these=10)

    def test_sanitize(self):
        orig = {'UserName': 'admin', 'Password': 'pwd',
                'nested': {'answer': 42, 'password': 'secret'}}
        expected = {'UserName': 'admin', 'Password': '***',
                    'nested': {'answer': 42, 'password': '***'}}
        self.assertEqual(expected, utils.sanitize(orig))


class ProcessApplyTimeTestCase(base.TestCase):

    def test_process_apply_time_input(self):
        payload = utils.process_apply_time_input(
            {'test': 'value'},
            sushy.ApplyTime.ON_RESET, None, None)
        self.assertEqual(
            {'@Redfish.SettingsApplyTime': {
                '@odata.type': '#Settings.v1_0_0.PreferredApplyTime',
                'ApplyTime': 'OnReset'},
                'test': 'value'}, payload)

    def test_process_apply_time_input_maintenance_window(self):
        payload = utils.process_apply_time_input(
            {'test': 'value'},
            sushy.ApplyTime.AT_MAINTENANCE_WINDOW_START,
            datetime.datetime(2020, 9, 1, 4, 30, 0),
            600)
        self.assertEqual(
            {'@Redfish.SettingsApplyTime': {
                '@odata.type': '#Settings.v1_0_0.PreferredApplyTime',
                'ApplyTime': 'AtMaintenanceWindowStart',
                'MaintenanceWindowDurationInSeconds': 600,
                'MaintenanceWindowStartTime': '2020-09-01T04:30:00'},
             'test': 'value'}, payload)

    def test_process_apply_time_missing(self):
        self.assertRaises(
            ValueError, utils.process_apply_time_input,
            {'test': 'value'}, None, datetime.datetime(2020, 9, 1, 4, 30, 0),
            600)

    def test_process_apply_time_maint_window_start_time_missing(self):
        self.assertRaises(
            ValueError, utils.process_apply_time_input, {'test': 'value'},
            sushy.ApplyTime.AT_MAINTENANCE_WINDOW_START, None, 600)

    def test_process_apply_time_maint_window_duration_missing(self):
        self.assertRaises(
            ValueError, utils.process_apply_time_input, {'test': 'value'},
            sushy.ApplyTime.AT_MAINTENANCE_WINDOW_START,
            datetime.datetime.now(), None)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

# This is referred from Redfish standard schema.
# http://redfish.dmtf.org/schemas/v1/Storage.v1_4_0.json

import logging

from sushy.resources import base
from sushy.resources import common
from sushy.resources import constants as res_cons
from sushy.resources.system.storage import constants
from sushy.resources.system.storage import controller
from sushy.resources.system.storage import drive
from sushy.resources.system.storage import volume
from sushy import utils


LOG = logging.getLogger(__name__)


class StorageControllersListField(base.ListField):
    """The set of storage controllers represented by this resource."""

    member_id = base.Field('MemberId', required=True)
    """Uniquely identifies the member within the collection."""

    name = base.Field('Name', required=True)
    """The name of the storage controller"""

    status = common.StatusField('Status')
    """Describes the status and health of the resource and its children."""

    identifiers = common.IdentifiersListField('Identifiers', default=[])
    """The Durable names for the storage controller."""

    speed_gbps = base.Field('SpeedGbps')
    """The maximum speed of the storage controller's device interface."""

    controller_protocols = base.MappedListField(
        'SupportedControllerProtocols', res_cons.Protocol)
    """The protocols by which this storage controller can be communicated to"""

    device_protocols = base.MappedListField('SupportedDeviceProtocols',
                                            res_cons.Protocol)
    """The protocols which the controller can use tocommunicate with devices"""

    raid_types = base.MappedListField('SupportedRAIDTypes', constants.RAIDType)
    """The set of RAID types supported by the storage controller."""


class Storage(base.ResourceBase):
    """This class represents the storage subsystem resources.

    A storage subsystem represents a set of storage controllers (physical or
    virtual) and the resources such as drives and volumes that can be accessed
    from that subsystem.
    """

    identity = base.Field('Id', required=True)
    """The Storage identity string"""

    name = base.Field('Name')
    """The name of the resource"""

    drives_identities = base.Field('Drives',
                                   adapter=utils.get_members_identities)
    """A tuple with the drive identities"""

    status = common.StatusField('Status')
    """Describes the status and health of the resource and its children."""

    def get_drive(self, drive_identity):
        """Given the drive identity return a ``Drive`` object

        :param drive_identity: The identity of the ``Drive``
        :returns: The ``Drive`` object
        :raises: ResourceNotFoundError
        """
        return drive.Drive(self._conn, drive_identity,
                           redfish_version=self.redfish_version,
                           registries=self.registries, root=self.root)

    @property
    @utils.cache_it
    def drives(self):
        """Return a list of `Drive` objects present in the storage resource.

        It is set once when the first time it is queried. On subsequent
        invocations, it returns a cached list of `Drives` objects until it is
        marked stale.

        :returns: A list of `Drive` objects
        :raises: ResourceNotFoundError
        """
        return [self.get_drive(id_) for id_ in self.drives_identities]

    @property
    @utils.cache_it
    def drives_sizes_bytes(self):
        """Sizes of all Drives in bytes in Storage resource.

        Returns the list of cached values until it (or its parent resource)
        is refreshed.
        """
        return sorted(drv.capacity_bytes for drv in self.drives)

    @property
    def drives_max_size_bytes(self):
        """Max size available in bytes among all Drives of this collection."""
        return utils.max_safe(self.drives_sizes_bytes)

    @property
    @utils.cache_it
    def volumes(self):
        """Property to reference `VolumeCollection` instance

        It is set once when the first time it is queried. On refresh,
        this property is marked as stale (greedy-refresh not done at that
        point). Here only the actual refresh of the sub-resource happens,
        if resource is stale.
        """
        return volume.VolumeCollection(
            self._conn, utils.get_sub_resource_path_by(self, 'Volumes'),
            redfish_version=self.redfish_version, root=self.root)

    storage_controllers = StorageControllersListField('StorageControllers',
                                                      default=[])
    """The storage devices associated with this resource.

    Deprecated since Redfish v1.13 to allow storage controllers be their own
    resource. Use `controllers` where available.
    """

    @property
    @utils.cache_it
    def controllers(self):
        """The storage controllers allocated to this storage subsystem.

        Replaces `storage_controllers` since Redfish v1.9 to allow storage
        controllers be their own resource.
        """
        return controller.ControllerCollection(
            self._conn,
            utils.get_sub_resource_path_by(self, "Controllers"),
            redfish_version=self.redfish_version,
            registries=self.registries, root=self.root)


class StorageCollection(base.ResourceCollectionBase):
    """This class represents the collection of Storage resources"""

    @property
    def _resource_type(self):
        return Storage

    @property
    @utils.cache_it
    def drives_sizes_bytes(self):
        """Sizes of each Drive in bytes in Storage collection resource.

        Returns the list of cached values until it (or its parent resource)
        is refreshed.
        """
        return sorted(drive_size for storage_ in self.get_members()
                      for drive_size in storage_.drives_sizes_bytes)

    @property
    def max_drive_size_bytes(self):
        """Max size available (in bytes) among all Drive resources.

        Returns the cached value until it (or its parent resource) is
        refreshed.
        """
        return utils.max_safe(self.drives_sizes_bytes)

    @property
    @utils.cache_it
    def volumes_sizes_bytes(self):
        """Sizes of each Volume in bytes in Storage collection resource.

        Returns the list of cached values until it (or its parent resource)
        is refreshed.
        """
        return sorted(volume_size for storage_ in self.get_members()
                      for volume_size in storage_.volumes.volumes_sizes_bytes)

    @property
    def max_volume_size_bytes(self):
        """Max size available (in bytes) among all Volume resources.

        Returns the cached value until it (or its parent resource) is
        refreshed.
        """
        return utils.max_safe(self.volumes_sizes_bytes)



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from http import client as http_client
import json
from unittest import mock

from sushy.resources import constants as res_cons
from sushy.resources.taskservice import constants as ts_cons
from sushy.resources.taskservice import task
from sushy.tests.unit import base


class TaskTestCase(base.TestCase):

    def setUp(self):
        super(TaskTestCase, self).setUp()
        self.conn = mock.Mock()
        with open('sushy/tests/unit/json_samples/task.json') as f:
            self.json_doc = json.load(f)

        self.conn.get.return_value.json.return_value = self.json_doc

        message_registry = mock.Mock()
        message = mock.Mock()
        message.message = "Property %1 is read only."
        message.number_of_args = 1
        message_registry.messages = {"PropertyNotWriteable": message}

        self.task = task.Task(
            self.conn, '/redfish/v1/TaskService/Tasks/545',
            redfish_version='1.4.3',
            registries={'Base.1.0': message_registry})

    def test__parse_attributes(self):
        self.task._parse_attributes(self.json_doc)
        self.assertEqual('545', self.task.identity)
        self.assertEqual('Task 545', self.task.name)
        self.assertEqual('Task description', self.task.description)
        self.assertEqual('/taskmon/545', self.task.task_monitor)
        self.assertEqual('2012-03-07T14:44+06:00', self.task.start_time)
        self.assertEqual('2012-03-07T14:45+06:00', self.task.end_time)
        self.assertEqual(100, self.task.percent_complete)
        self.assertEqual(ts_cons.TaskState.COMPLETED, self.task.task_state)
        self.assertEqual(res_cons.Health.OK, self.task.task_status)
        self.assertEqual(1, len(self.task.messages))
        self.assertEqual('Base.1.0.PropertyNotWriteable',
                         self.task.messages[0].message_id)
        self.assertEqual('Property %1 is read only.',
                         self.task.messages[0].message)
        self.assertEqual(res_cons.Severity.WARNING,
                         self.task.messages[0].severity)

    def test_is_processing_true(self):
        self.task.status_code = http_client.ACCEPTED
        self.assertTrue(self.task.is_processing)

    def test_is_processing_false(self):
        self.task.status_code = http_client.OK
        self.assertFalse(self.task.is_processing)

    def test_parse_messages(self):
        self.task.parse_messages()
        self.assertEqual('Property SKU is read only.',
                         self.task.messages[0].message)


class TaskCollectionTestCase(base.TestCase):

    def setUp(self):
        super(TaskCollectionTestCase, self).setUp()
        self.conn = mock.Mock()
        with open('sushy/tests/unit/json_samples/'
                  'task_collection.json') as f:
            self.json_doc = json.load(f)

        self.conn.get.return_value.json.return_value = self.json_doc

        self.task_col = task.TaskCollection(
            self.conn, '/redfish/v1/TaskService/Tasks',
            redfish_version='1.0.2')

    def test__parse_attributes(self):
        self.task_col._parse_attributes(self.json_doc)
        self.assertEqual('1.0.2', self.task_col.redfish_version)
        self.assertEqual('Task Collection', self.task_col.name)
        self.assertEqual(('/redfish/v1/TaskService/Tasks/545',
                          '/redfish/v1/TaskService/Tasks/546'),
                         self.task_col.members_identities)

    @mock.patch.object(task, 'Task', autospec=True)
    def test_get_member(self, mock_task):
        self.task_col.get_member(
            '/redfish/v1/TaskService/Tasks/545')
        mock_task.assert_called_once_with(
            self.task_col._conn,
            '/redfish/v1/TaskService/Tasks/545',
            redfish_version=self.task_col.redfish_version, registries=None,
            root=self.task_col.root)

    @mock.patch.object(task, 'Task', autospec=True)
    def test_get_members(self, mock_task):
        members = self.task_col.get_members()
        calls = [
            mock.call(self.task_col._conn,
                      '/redfish/v1/TaskService/Tasks/545',
                      redfish_version=self.task_col.redfish_version,
                      registries=None,
                      root=self.task_col.root),
            mock.call(self.task_col._conn,
                      '/redfish/v1/TaskService/Tasks/546',
                      redfish_version=self.task_col.redfish_version,
                      registries=None,
                      root=self.task_col.root),
        ]
        mock_task.assert_has_calls(calls)
        self.assertIsInstance(members, list)
        self.assertEqual(2, len(members))

    def _setUp_task_summary(self):
        self.conn.get.return_value.json.reset_mock()
        successive_return_values = []
        file_names = ['sushy/tests/unit/json_samples/task.json',
                      'sushy/tests/unit/json_samples/task2.json']
        for file_name in file_names:
            with open(file_name) as f:
                successive_return_values.append(json.load(f))

        self.conn.get.return_value.json.side_effect = successive_return_values

    def test_summary(self):
        # | GIVEN |
        self._setUp_task_summary()
        # | WHEN |
        actual_summary = self.task_col.summary
        # | THEN |
        self.assertEqual({'545': ts_cons.TaskState.COMPLETED,
                          '546': ts_cons.TaskState.PENDING},
                         actual_summary)

        # reset mock
        self.conn.get.return_value.json.reset_mock()

        # | WHEN & THEN |
        # tests for same object on invoking subsequently
        self.assertIs(actual_summary,
                      self.task_col.summary)
        self.conn.get.return_value.json.assert_not_called()

    def test_summary_on_refresh(self):
        # | GIVEN |
        self._setUp_task_summary()
        # | WHEN & THEN |
        self.assertEqual({'545': ts_cons.TaskState.COMPLETED,
                          '546': ts_cons.TaskState.PENDING},
                         self.task_col.summary)

        self.conn.get.return_value.json.side_effect = None
        # On refreshing the task_col instance...
        with open('sushy/tests/unit/json_samples/'
                  'task_collection.json') as f:
            self.conn.get.return_value.json.return_value = json.load(f)
        self.task_col.invalidate()
        self.task_col.refresh(force=False)

        # | GIVEN |
        self._setUp_task_summary()
        # | WHEN & THEN |
        self.assertEqual({'545': ts_cons.TaskState.COMPLETED,
                          '546': ts_cons.TaskState.PENDING},
                         self.task_col.summary)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

# This is referred from Redfish standard schema.
# https://redfish.dmtf.org/schemas/Bios.v1_0_3.json

from http import client as http_client
import logging

from sushy import exceptions
from sushy.resources import base
from sushy.resources import common
from sushy.resources import settings
from sushy import utils

LOG = logging.getLogger(__name__)


class ActionsField(base.CompositeField):
    change_password = common.ActionField('#Bios.ChangePassword')
    reset_bios = common.ActionField('#Bios.ResetBios')


class Bios(base.ResourceBase):

    def __init__(self, connector, path, redfish_version=None, registries=None,
                 root=None):
        """A class representing a Bios

        :param connector: A Connector instance
        :param path: Sub-URI path to the Bios resource
        :param registries: Dict of message registries to be used when
            parsing messages of attribute update status
        :param root: Sushy root object. Empty for Sushy root itself.
        """
        super(Bios, self).__init__(
            connector, path, redfish_version=redfish_version,
            registries=registries, root=root)

    identity = base.Field('Id', required=True)
    """The Bios resource identity string"""

    name = base.Field('Name')
    """The name of the resource"""

    description = base.Field('Description')
    """Human-readable description of the BIOS resource"""

    _attribute_registry = base.Field('AttributeRegistry')
    """The Resource ID of the Attribute Registry
    for the BIOS Attributes resource
    """

    _settings = settings.SettingsField()
    """Results of last BIOS attribute update"""

    attributes = base.Field('Attributes')
    """Vendor-specific key-value dict of effective BIOS attributes

    Attributes cannot be updated directly.
    To update use :py:func:`~set_attribute` or :py:func:`~set_attributes`
    """

    maintenance_window = settings.MaintenanceWindowField(
        '@Redfish.MaintenanceWindow')
    """Indicates if a given resource has a maintenance window assignment
    for applying settings or operations"""

    _actions = ActionsField('Actions')

    _apply_time_settings = settings.SettingsApplyTimeField()

    @property
    @utils.cache_it
    def _pending_settings_resource(self):
        """Pending BIOS settings resource"""
        return Bios(
            self._conn, self._settings.resource_uri,
            registries=None,
            redfish_version=self.redfish_version, root=self.root)

    @property
    def pending_attributes(self):
        """Pending BIOS attributes

        BIOS attributes that have been committed to the system,
        but for them to take effect system restart is necessary
        """
        return self._pending_settings_resource.attributes

    @property
    def apply_time_settings(self):
        return self._pending_settings_resource._apply_time_settings

    def set_attribute(self, key, value, apply_time=None,
                      maint_window_start_time=None,
                      maint_window_duration=None):
        """Update an attribute

        Attribute update is not immediate but requires system restart.
        Committed attributes can be checked at :py:attr:`~pending_attributes`
        property

        :param key: Attribute name
        :param value: Attribute value
        :param apply_time: When to update the attribute. Optional.
            An :py:class:`sushy.ApplyTime` value.
        :param maint_window_start_time: The start time of a maintenance window,
            datetime. Required when updating during maintenance window and
            default maintenance window not set by the system.
        :param maint_window_duration: Duration of maintenance time since
            maintenance window start time in seconds. Required when updating
            during maintenance window and default maintenance window not
            set by the system.
        """
        self.set_attributes({key: value}, apply_time, maint_window_start_time,
                            maint_window_duration)

    def set_attributes(self, value, apply_time=None,
                       maint_window_start_time=None,
                       maint_window_duration=None):
        """Update many attributes at once

        Attribute update is not immediate but requires system restart.
        Committed attributes can be checked at :py:attr:`~pending_attributes`
        property

        :param value: Key-value pairs for attribute name and value
        :param apply_time: When to update the attributes. Optional.
            An :py:class:`sushy.ApplyTime` value.
        :param maint_window_start_time: The start time of a maintenance window,
            datetime. Required when updating during maintenance window and
            default maintenance window not set by the system.
        :param maint_window_duration: Duration of maintenance time since
            maintenance window start time in seconds. Required when updating
            during maintenance window and default maintenance window not
            set by the system.
        """
        payload = {'Attributes': value}
        payload = utils.process_apply_time_input(
            payload, apply_time, maint_window_start_time,
            maint_window_duration)
        self._settings.commit(self._conn,
                              payload)
        utils.cache_clear(self, force_refresh=False,
                          only_these=['_pending_settings_resource'])

    def _get_reset_bios_action_element(self):
        actions = self._actions

        if not actions:
            raise exceptions.MissingAttributeError(attribute="Actions",
                                                   resource=self._path)

        reset_bios_action = actions.reset_bios

        if not reset_bios_action:
            raise exceptions.MissingActionError(action='#Bios.ResetBios',
                                                resource=self._path)
        return reset_bios_action

    def _get_change_password_element(self):
        actions = self._actions

        if not actions:
            raise exceptions.MissingAttributeError(attribute="Actions",
                                                   resource=self._path)

        change_password_action = actions.change_password

        if not change_password_action:
            raise exceptions.MissingActionError(action='#Bios.ChangePassword',
                                                resource=self._path)
        return change_password_action

    def reset_bios(self):
        """Reset the BIOS attributes to default"""

        target_uri = self._get_reset_bios_action_element().target_uri

        LOG.debug('Resetting BIOS attributes %s ...', self.identity)
        try:
            self._conn.post(target_uri)
        except exceptions.HTTPError as resp:
            # Send empty payload, if BMC expects body
            if resp.status_code in [http_client.UNSUPPORTED_MEDIA_TYPE,
                                    http_client.BAD_REQUEST]:
                self._conn.post(target_uri, data={})
            else:
                raise

        LOG.info('BIOS attributes %s is being reset', self.identity)

    def change_password(self, new_password, old_password, password_name):
        """Change BIOS password"""

        target_uri = self._get_change_password_element().target_uri

        LOG.debug('Changing BIOS password %s ...', self.identity)
        self._conn.post(target_uri, data={'NewPassword': new_password,
                                          'OldPassword': old_password,
                                          'PasswordName': password_name})
        LOG.info('BIOS password %s is being changed', self.identity)

    @property
    def update_status(self):
        """Status of the last attribute update

        :returns: :class:`sushy.resources.settings.SettingsUpdate` object
            containing status and any messages
        """
        return self._settings.get_status(self._registries)

    @property
    def supported_apply_times(self):
        """List of supported BIOS update apply times

        :returns: List of supported update apply time names
        """
        return self._settings._supported_apply_times

    def get_attribute_registry(self, language='en'):
        """Get the Attribute Registry associated with this BIOS instance

        :param language: RFC 5646 language code for Message Registries.
            Indicates language of registry to be used. Defaults to 'en'.
        :returns: the BIOS Attribute Registry
        """
        return self._get_registry(self._attribute_registry,
                                  language=language,
                                  description='BIOS attribute registry')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

# Values come from the Redfish UpdateService json-schema.
# https://redfish.dmtf.org/schemas/v1/TaskService.v1_1_5.json#/definitions/OverWritePolicy

import enum


class TaskState(enum.Enum):
    """Task state related constants."""
    CANCELLED = 'Cancelled'
    CANCELLING = 'Cancelling'
    COMPLETED = 'Completed'
    EXCEPTION = 'Exception'
    INTERRUPTED = 'Interrupted'
    NEW = 'New'
    PENDING = 'Pending'
    RUNNING = 'Running'
    SERVICE = 'Service'
    STARTING = 'Starting'
    STOPPING = 'Stopping'
    SUSPENDED = 'Suspended'

    # Deprecated in 1.2.0
    KILLED = 'Killed'


class OverWritePolicy(enum.Enum):
    """Overwrite Policy constants"""

    MANUAL = 'Manual'
    """Completed tasks are not automatically overwritten."""

    OLDEST = 'Oldest'
    """Oldest completed tasks are overwritten."""


# Backward compatibility
OVERWRITE_POLICY_MANUAL = OverWritePolicy.MANUAL
OVERWRITE_POLICY_OLDEST = OverWritePolicy.OLDEST



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from sushy import exceptions
from sushy.resources import base
from sushy.resources.certificateservice import certificate
from sushy.resources.certificateservice import constants
from sushy.resources import common
from sushy import utils


class ActionsField(base.CompositeField):
    generate_csr = common.ActionField('#CertificateService.GenerateCSR')
    replace_certificate = common.ActionField(
        '#CertificateService.ReplaceCertificate')


class CertificateLocations(base.ResourceLinksBase):

    name = base.Field('Name')
    """The name of the collection"""

    _resource_type = certificate.Certificate

    @property
    def members_identities(self):
        return utils.get_sub_resource_path_by(
            self, ["Links", "Certificates"], is_collection=True)


class CertificateService(base.ResourceBase):

    identity = base.Field('Id', required=True)
    """The certificate service identity"""

    name = base.Field('Name')
    """The certificate service name"""

    _actions = ActionsField('Actions')

    _certificate_locations_path = base.Field(
        ['CertificateLocations', '@odata.id'])

    def _get_replace_certificate_action_element(self):
        reset_action = self._actions.replace_certificate
        if not reset_action:
            raise exceptions.MissingActionError(
                action='#CertificateService.ReplaceCertificate',
                resource=self._path)
        return reset_action

    @property
    @utils.cache_it
    def certificate_locations(self):
        """Property to reference certificate locations instance"""
        if not self._certificate_locations_path:
            raise exceptions.MissingAttributeError(
                attribute='CertificateLocations/@odata.id',
                resource=self._path)

        return CertificateLocations(
            self._conn,
            self._certificate_locations_path,
            redfish_version=self.redfish_version,
            registries=self.registries, root=self.root)

    def replace_certificate(self, certificate_uri, certificate_string,
                            certificate_type):
        """Replace an existing certificate in the service.

        :param certificate_uri: URI of an existing certificate.
        :param certificate_string: the contents of the new certificate.
        :param certificate_type: the type of the new certificate, one of
            :py:class:`sushy.CertificateType`.
        """
        try:
            certificate_type = constants.CertificateType(certificate_type)
        except ValueError:
            raise exceptions.InvalidParameterValueError(
                parameter='certificate_type',
                value=certificate_type,
                valid_values=list(constants.CertificateType))

        target_uri = self._get_replace_certificate_action_element().target_uri

        body = {'CertificateUri': certificate_uri,
                'CertificateString': certificate_string,
                'CertificateType': certificate_type.value}
        self._conn.post(target_uri, data=body)



# Copyright (c) 2021 Dell, Inc. or its subsidiaries
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from datetime import datetime
from http import client as http_client
import logging
import time
from urllib.parse import urljoin

from dateutil import parser

from sushy import exceptions
from sushy.resources.taskservice import task

LOG = logging.getLogger(__name__)


class TaskMonitor(object):
    def __init__(self,
                 connector,
                 task_monitor_uri,
                 redfish_version=None,
                 registries=None,
                 response=None):
        """A class representing a task monitor

        :param connector: A Connector instance
        :param task_monitor_uri: The task monitor URI
        :param redfish_version: The version of Redfish. Used to construct
            the object according to schema of the given version.
        :param registries: Dict of Redfish Message Registry objects to be
            used in any resource that needs registries to parse messages.
        :param response: Raw response
        """
        self._connector = connector
        self._task_monitor_uri = task_monitor_uri
        self._redfish_version = redfish_version
        self._registries = registries
        self._task = None
        self._response = response

        if (self._response and self._response.content
                and self._response.status_code == http_client.ACCEPTED):
            self._task = task.Task(self._connector, self._task_monitor_uri,
                                   redfish_version=self._redfish_version,
                                   registries=self._registries,
                                   json_doc=self._response.json())
        else:
            self.refresh()

    def refresh(self):
        """Refresh the Task

        Freshly retrieves/fetches the Task.
        :raises: ResourceNotFoundError
        :raises: ConnectionError
        :raises: HTTPError
        """
        self._response = self._connector.get(path=self.task_monitor_uri)

        if self._response.status_code == http_client.ACCEPTED:
            # A Task should have been returned, but wasn't
            if not self._response.content:
                self._task = None
                return

            # Assume that the body contains a Task since we got a 202
            if not self._task:
                self._task = task.Task(self._connector, self._task_monitor_uri,
                                       redfish_version=self._redfish_version,
                                       registries=self._registries,
                                       json_doc=self._response.json())
            else:
                self._task.refresh(json_doc=self._response.json())
        else:
            self._task = None

    @property
    def task_monitor_uri(self):
        """The TaskMonitor URI

        :returns: The TaskMonitor URI.
        """
        return self._task_monitor_uri

    @property
    def is_processing(self):
        """Indicates if the task is still processing

        :returns: A boolean indicating if the task is still processing.
        """
        return self._response.status_code == http_client.ACCEPTED

    @property
    def check_is_processing(self):
        """Refreshes task and check if it is still processing

        :returns: A boolean indicating if the task is still processing.
        """
        if not self.is_processing:
            return False

        self.refresh()

        return self.is_processing

    @property
    def sleep_for(self):
        """Seconds the client should wait before querying the operation status

        Defaults to 1 second if Retry-After not specified in response.

        :returns: The number of seconds to wait
        """
        retry_after = self._response.headers.get('Retry-After')
        if retry_after is None:
            return 1

        if isinstance(retry_after, int) or retry_after.isdigit():
            return retry_after

        return max(0, (parser.parse(retry_after)
                   - datetime.now().astimezone()).total_seconds())

    @property
    def cancellable(self):
        """The amount of time to sleep before retrying

        :returns: A Boolean indicating if the Task is cancellable.
        """
        allow = self._response.headers.get('Allow')

        cancellable = False
        if allow and allow.upper() == 'DELETE':
            cancellable = True

        return cancellable

    @property
    def task(self):
        """The executing task

        :returns: The Task being executed.
        """

        return self._task

    @property
    def response(self):
        """Unprocessed response.

        Intended to be used internally.
        :returns: Unprocessed response.
        """
        return self._response

    def get_task(self):
        """Construct Task instance from task monitor URI.

        :returns: Task instance.
        """
        return task.Task(self._connector, self._task_monitor_uri,
                         redfish_version=self._redfish_version,
                         registries=self._registries)

    def wait(self, timeout_sec):
        """Waits until task is completed or it times out.

        :param timeout_sec: Timeout to wait
        :raises: ConnectionError when times out
        """
        timeout_at = time.time() + timeout_sec

        while self.check_is_processing:

            LOG.debug('Waiting for task monitor %(url)s; sleeping for '
                      '%(sleep)s seconds',
                      {'url': self.task_monitor_uri,
                       'sleep': self.sleep_for})
            time.sleep(self.sleep_for)
            if time.time() >= timeout_at and self.check_is_processing:
                m = ('Timeout waiting for task monitor %(url)s '
                     '(timeout = %(timeout)s)'
                     % {'url': self.task_monitor_uri,
                        'timeout': timeout_sec})
                raise exceptions.ConnectionError(url=self.task_monitor_uri,
                                                 error=m)

    @staticmethod
    def from_response(conn, response, target_uri, redfish_version=None,
                      registries=None):
        """Construct TaskMonitor instance from received response.

        :response: Unprocessed response
        :target_uri: URI used to initiate async operation
        :redfish_version: Redfish version. Optional when used internally.
        :registries: Redfish registries. Optional when used internally.
        :returns: TaskMonitor instance
        :raises: MissingHeaderError if Location is missing in response
        """
        json_data = response.json() if response.content else {}

        header = 'Location'
        task_monitor_uri = response.headers.get(header)
        task_uri_data = json_data.get('@odata.id')

        if task_uri_data:
            task_monitor_uri = urljoin(task_monitor_uri, task_uri_data)

        if not task_monitor_uri:
            raise exceptions.MissingHeaderError(target_uri=target_uri,
                                                header=header)

        return TaskMonitor(conn,
                           task_monitor_uri,
                           redfish_version=redfish_version,
                           registries=registries,
                           response=response)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

# This is referred from Redfish standard schema.
# https://redfish.dmtf.org/schemas/Manager.v1_4_0.json

import logging

from sushy import exceptions
from sushy.resources import base
from sushy.resources import common
from sushy.resources import constants as res_cons
from sushy.resources.manager import constants as mgr_cons
from sushy.resources.manager import virtual_media
from sushy import utils


LOG = logging.getLogger(__name__)


class ActionsField(base.CompositeField):
    reset = common.ResetActionField('#Manager.Reset')


class RemoteAccessField(base.CompositeField):
    service_enabled = base.Field('ServiceEnabled')

    max_concurrent_sessions = base.Field('MaxConcurrentSessions')

    connect_types_supported = base.Field('ConnectTypesSupported',
                                         adapter=list)


class Manager(base.ResourceBase):

    auto_dst_enabled = base.Field('AutoDSTEnabled')
    """Indicates whether the manager is configured for automatic DST
        adjustment"""

    firmware_version = base.Field('FirmwareVersion')
    """The manager firmware version"""

    graphical_console = RemoteAccessField('GraphicalConsole')
    """A dictionary containing the remote access support service via
       graphical console (e.g. KVMIP) and max concurrent sessions
    """

    serial_console = RemoteAccessField('SerialConsole')
    """A dictionary containing the remote access support service via
       serial console (e.g. Telnet, SSH, IPMI) and max concurrent sessions
    """

    command_shell = RemoteAccessField('CommandShell')
    """A dictionary containing the remote access support service via
       command shell (e.g. Telnet, SSH) and max concurrent sessions
    """

    description = base.Field('Description')
    """The manager description"""

    identity = base.Field('Id', required=True)
    """The manager identity string"""

    name = base.Field('Name')
    """The manager name"""

    model = base.Field('Model')
    """The manager model"""

    manager_type = base.MappedField('ManagerType', mgr_cons.ManagerType)
    """The manager type"""

    uuid = base.Field('UUID')
    """The manager UUID"""

    _actions = ActionsField('Actions')

    def __init__(self, connector, identity, redfish_version=None,
                 registries=None, root=None):
        """A class representing a Manager

        :param connector: A Connector instance
        :param identity: The identity of the Manager resource
        :param redfish_version: The version of RedFish. Used to construct
            the object according to schema of the given version.
        :param registries: Dict of Redfish Message Registry objects to be
            used in any resource that needs registries to parse messages
        :param root: Sushy root object. Empty for Sushy root itself.
        """
        super(Manager, self).__init__(
            connector, identity, redfish_version=redfish_version,
            registries=registries, root=root)

    def get_supported_graphical_console_types(self):
        """Get the supported values for Graphical Console connection types.

        :returns: A set of supported values.
        """
        if (not self.graphical_console
                or not self.graphical_console.connect_types_supported):
            LOG.warning('Could not figure out the supported values for '
                        'remote access via graphical console for Manager %s',
                        self.identity)
            return set(mgr_cons.GraphicalConnectType)

        return {v for v in mgr_cons.GraphicalConnectType
                if v.value in self.graphical_console.connect_types_supported}

    def get_supported_serial_console_types(self):
        """Get the supported values for Serial Console connection types.

        :returns: A set of supported values.
        """
        if (not self.serial_console
                or not self.serial_console.connect_types_supported):
            LOG.warning('Could not figure out the supported values for '
                        'remote access via serial console for Manager %s',
                        self.identity)
            return set(mgr_cons.SerialConnectType)

        return {v for v in mgr_cons.SerialConnectType
                if v.value in self.serial_console.connect_types_supported}

    def get_supported_command_shell_types(self):
        """Get the supported values for Command Shell connection types.

        :returns: A set of supported values.
        """
        if (not self.command_shell
                or not self.command_shell.connect_types_supported):
            LOG.warning('Could not figure out the supported values for '
                        'remote access via command shell for Manager %s',
                        self.identity)
            return set(mgr_cons.CommandConnectType)

        return {v for v in mgr_cons.CommandConnectType
                if v.value in self.command_shell.connect_types_supported}

    def _get_reset_action_element(self):
        reset_action = self._actions.reset

        if not reset_action:
            raise exceptions.MissingActionError(action='#Manager.Reset',
                                                resource=self._path)
        return reset_action

    def get_allowed_reset_manager_values(self):
        """Get the allowed values for resetting the manager.

        :returns: A set of allowed values.
        :raises: MissingAttributeError, if Actions/#Manager.Reset attribute
            not present.
        """
        reset_action = self._get_reset_action_element()

        if not reset_action.allowed_values:
            LOG.warning('Could not figure out the allowed values for the '
                        'reset manager action for Manager %s', self.identity)
            return set(res_cons.ResetType)

        return {v for v in res_cons.ResetType
                if v.value in reset_action.allowed_values}

    def reset_manager(self, value):
        """Reset the manager.

        :param value: The target value.
        :raises: InvalidParameterValueError, if the target value is not
            allowed.
        """
        valid_resets = self.get_allowed_reset_manager_values()
        if value not in valid_resets:
            raise exceptions.InvalidParameterValueError(
                parameter='value', value=value, valid_values=valid_resets)

        value = res_cons.ResetType(value).value
        target_uri = self._get_reset_action_element().target_uri

        LOG.debug('Resetting the Manager %s ...', self.identity)
        self._conn.post(target_uri, data={'ResetType': value})
        LOG.info('The Manager %s is being reset', self.identity)

    @property
    @utils.cache_it
    def virtual_media(self):
        return virtual_media.VirtualMediaCollection(
            self._conn, utils.get_sub_resource_path_by(self, 'VirtualMedia'),
            redfish_version=self.redfish_version, registries=self.registries,
            root=self.root)

    @property
    @utils.cache_it
    def systems(self):
        """A list of systems managed by this manager.

        Returns a list of `System` objects representing systems being
        managed by this manager.

        :raises: MissingAttributeError if '@odata.id' field is missing.
        :returns: A list of `System` instances
        """
        paths = utils.get_sub_resource_path_by(
            self, ["Links", "ManagerForServers"], is_collection=True)

        from sushy.resources.system import system
        return [system.System(self._conn, path,
                              redfish_version=self.redfish_version,
                              registries=self.registries, root=self.root)
                for path in paths]

    @property
    @utils.cache_it
    def chassis(self):
        """A list of chassis managed by this manager.

        Returns a list of `Chassis` objects representing the chassis
        or cabinets managed by this manager.

        :raises: MissingAttributeError if '@odata.id' field is missing.
        :returns: A list of `Chassis` instances
        """
        paths = utils.get_sub_resource_path_by(
            self, ["Links", "ManagerForChassis"], is_collection=True)

        from sushy.resources.chassis import chassis
        return [chassis.Chassis(self._conn, path,
                                redfish_version=self.redfish_version,
                                registries=self.registries, root=self.root)
                for path in paths]


class ManagerCollection(base.ResourceCollectionBase):

    @property
    def _resource_type(self):
        return Manager

    def __init__(self, connector, path, redfish_version=None, registries=None,
                 root=None):
        """A class representing a ManagerCollection

        :param connector: A Connector instance
        :param path: The canonical path to the Manager collection resource
        :param redfish_version: The version of RedFish. Used to construct
            the object according to schema of the given version.
        :param registries: Dict of Redfish Message Registry objects to be
            used in any resource that needs registries to parse messages
        :param root: Sushy root object. Empty for Sushy root itself.
        """
        super(ManagerCollection, self).__init__(
            connector, path, redfish_version=redfish_version,
            registries=registries, root=root)



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import json
from unittest import mock

from sushy.resources import constants as res_cons
from sushy.resources.taskservice import constants as ts_cons
from sushy.resources.taskservice import task
from sushy.resources.taskservice import taskservice
from sushy.tests.unit import base


class TaskServiceTestCase(base.TestCase):

    def setUp(self):
        super(TaskServiceTestCase, self).setUp()
        self.conn = mock.Mock()
        with open('sushy/tests/unit/json_samples/taskservice.json') as f:
            self.json_doc = json.load(f)

        self.conn.get.return_value.json.return_value = self.json_doc

        self.tsk_serv = taskservice.TaskService(
            self.conn, '/redfish/v1/TaskService/TaskService',
            redfish_version='1.3.0')

    def test__parse_attributes(self):
        self.tsk_serv._parse_attributes(self.json_doc)
        self.assertEqual('TaskService', self.tsk_serv.identity)
        self.assertTrue(self.tsk_serv.service_enabled)
        self.assertTrue(self.tsk_serv.event_on_task_state_change)
        self.assertEqual(res_cons.State.ENABLED, self.tsk_serv.status.state)
        self.assertEqual(res_cons.Health.OK, self.tsk_serv.status.health)
        self.assertEqual(self.tsk_serv.overwrite_policy,
                         ts_cons.OverWritePolicy.MANUAL)

    @mock.patch.object(task, 'TaskCollection', autospec=True)
    def test_tasks(self, task_collection_mock):
        self.tsk_serv.tasks
        task_collection_mock.assert_called_once_with(
            self.conn, '/redfish/v1/TaskService/Tasks',
            self.tsk_serv.redfish_version,
            self.tsk_serv._registries,
            self.tsk_serv.root)



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# This is referred from Redfish standard schema.
# https://redfish.dmtf.org/schemas/Zone.v1_2_0.json

import logging

from sushy.resources import base
from sushy.resources import common

LOG = logging.getLogger(__name__)


class LinksField(base.CompositeField):

    endpoints = base.Field('Endpoints')
    """The references to the endpoints that are contained in this zone"""

    involved_switches = base.Field('InvolvedSwitches')
    """The references to the switches in this zone"""

    resource_blocks = base.Field('ResourceBlocks')
    """The references to the Resource Blocks that are used in this zone"""


class ResourceZone(base.ResourceBase):

    # Note(dnuka): This patch doesn't contain 100% of the ResourceZone

    description = base.Field('Description')
    """The resources zone description"""

    identity = base.Field('Id', required=True)
    """The resource zone identity string"""

    links = LinksField('Links')
    """The references to other resources that are related to this
    resource"""

    name = base.Field('Name', required=True)
    """The resource zone name"""

    status = common.StatusField('Status')
    """The resource zone status"""

    def __init__(self, connector, identity, redfish_version=None,
                 registries=None, root=None):
        """A class representing a ResourceZone

        :param connector: A Connector instance
        :param identity: The identity of the ResourceZone resource
        :param redfish_version: The version of RedFish. Used to construct
            the object according to schema of given version.
        :param registries: Dict of Redfish Message Registry objects to be
            used in any resource that needs registries to parse messages
        :param root: Sushy root object. Empty for Sushy root itself.
        """
        super(ResourceZone, self).__init__(
            connector, identity, redfish_version=redfish_version,
            registries=registries, root=root)


class ResourceZoneCollection(base.ResourceCollectionBase):

    name = base.Field('Name')
    """The resource zone collection name"""

    description = base.Field('Description')
    """The resource zone collection description"""

    @property
    def _resource_type(self):
        return ResourceZone

    def __init__(self, connector, identity, redfish_version=None,
                 registries=None, root=None):
        """A class representing a ResourceZoneCollection

        :param connector: A Connector instance
        :param identity: The identity of the ResourceZone resource
        :param redfish_version: The version of RedFish. Used to construct
            the object according to schema of given version.
        :param registries: Dict of Redfish Message Registry objects to be
            used in any resource that needs registries to parse messages
        :param root: Sushy root object. Empty for Sushy root itself.
        """
        super(ResourceZoneCollection, self).__init__(
            connector, identity, redfish_version=redfish_version,
            registries=registries, root=root)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

# This is referred from Redfish standard schema.
# http://redfish.dmtf.org/schemas/v1/Chassis.v1_8_0.json

import logging

from sushy import exceptions
from sushy.resources import base
from sushy.resources.chassis import constants as cha_cons
from sushy.resources.chassis.power import power
from sushy.resources.chassis.thermal import thermal
from sushy.resources import common
from sushy.resources import constants as res_cons
from sushy.resources.manager import manager
from sushy.resources.system.network import adapter
from sushy import utils

LOG = logging.getLogger(__name__)


class ActionsField(base.CompositeField):
    reset = common.ResetActionField('#Chassis.Reset')


class PhysicalSecurity(base.CompositeField):
    intrusion_sensor = base.MappedField('IntrusionSensor',
                                        cha_cons.IntrusionSensor)
    """IntrusionSensor
    This indicates the known state of the physical security sensor, such as if
    it is hardware intrusion detected.
    """

    intrusion_sensor_number = base.Field('IntrusionSensorNumber')
    """A numerical identifier to represent the physical security sensor"""

    intrusion_sensor_re_arm = base.MappedField('IntrusionSensorReArm',
                                               cha_cons.IntrusionSensorReArm)
    """This indicates how the Normal state to be restored"""


class Chassis(base.ResourceBase):
    """Chassis resource

    The Chassis represents the physical components of a system. This
    resource represents the sheet-metal confined spaces and logical zones
    such as racks, enclosures, chassis and all other containers.
    """

    chassis_type = base.MappedField('ChassisType', cha_cons.ChassisType,
                                    required=True)
    """The type of physical form factor of the chassis"""

    identity = base.Field('Id', required=True)
    """Identifier for the chassis"""

    name = base.Field('Name', required=True)
    """The chassis name"""

    asset_tag = base.Field('AssetTag')
    """The user assigned asset tag of this chassis"""

    depth_mm = base.Field('DepthMm')
    """Depth in millimeters
    The depth of the chassis. The value of this property shall represent
    the depth (length) of the chassis (in millimeters) as specified by the
    manufacturer.
    """

    description = base.Field('Description')
    """The chassis description"""

    height_mm = base.Field('HeightMm')
    """Height in millimeters
    The height of the chassis. The value of this property shall represent
    the height of the chassis (in millimeters) as specified by the
    manufacturer.
    """

    indicator_led = base.MappedField('IndicatorLED', res_cons.IndicatorLED)
    """The state of the indicator LED, used to identify the chassis"""

    manufacturer = base.Field('Manufacturer')
    """The manufacturer of this chassis"""

    model = base.Field('Model')
    """The model number of the chassis"""

    part_number = base.Field('PartNumber')
    """The part number of the chassis"""

    physical_security = PhysicalSecurity('PhysicalSecurity')
    """PhysicalSecurity
    This value of this property shall contain the sensor state of the physical
    security.
    """

    power_state = base.MappedField('PowerState', res_cons.PowerState)
    """The current power state of the chassis"""

    serial_number = base.Field('SerialNumber')
    """The serial number of the chassis"""

    sku = base.Field('SKU')
    """Stock-keeping unit number (SKU)
    The value of this property shall be the stock-keeping unit number for
    this chassis.
    """

    status = common.StatusField('Status')
    """Status and Health
    This property describes the status and health of the chassis and its
    children.
    """

    uuid = base.Field('UUID')
    """The Universal Unique Identifier (UUID) for this Chassis."""

    weight_kg = base.Field('WeightKg')
    """Weight in kilograms
    The value of this property shall represent the published mass (commonly
    referred to as weight) of the chassis (in kilograms).
    """

    width_mm = base.Field('WidthMm')
    """Width in millimeters
    The value of this property shall represent the width of the chassis
    (in millimeters) as specified by the manufacturer.
    """

    _actions = ActionsField('Actions')

    def __init__(self, connector, identity, redfish_version=None,
                 registries=None, root=None):
        """A class representing a Chassis

        :param connector: A Connector instance
        :param identity: The identity of the Chassis resource
        :param redfish_version: The version of RedFish. Used to construct
            the object according to schema of the given version.
        :param registries: Dict of Redfish Message Registry objects to be
            used in any resource that needs registries to parse messages
        :param root: Sushy root object. Empty for Sushy root itself.
        """
        super(Chassis, self).__init__(
            connector, identity, redfish_version=redfish_version,
            registries=registries, root=root)

    def _get_reset_action_element(self):
        reset_action = self._actions.reset

        if not reset_action:
            raise exceptions.MissingActionError(action='#Chassis.Reset',
                                                resource=self._path)
        return reset_action

    def get_allowed_reset_chassis_values(self):
        """Get the allowed values for resetting the chassis.

        :returns: A set of allowed values.
        :raises: MissingAttributeError, if Actions/#Chassis.Reset attribute
            not present.
        """
        reset_action = self._get_reset_action_element()

        if not reset_action.allowed_values:
            LOG.warning('Could not figure out the allowed values for the '
                        'reset chassis action for Chassis %s', self.identity)
            return set(res_cons.ResetType)

        return {v for v in res_cons.ResetType
                if v.value in reset_action.allowed_values}

    def reset_chassis(self, value):
        """Reset the chassis.

        :param value: The target value.
        :raises: InvalidParameterValueError, if the target value is not
            allowed.
        """
        valid_resets = self.get_allowed_reset_chassis_values()
        if value not in valid_resets:
            raise exceptions.InvalidParameterValueError(
                parameter='value', value=value, valid_values=valid_resets)

        value = res_cons.ResetType(value).value
        target_uri = self._get_reset_action_element().target_uri

        LOG.debug('Resetting the Chassis %s ...', self.identity)
        self._conn.post(target_uri, data={'ResetType': value})
        LOG.info('The Chassis %s is being reset', self.identity)

    def set_indicator_led(self, state):
        """Set IndicatorLED to the given state.

        :param state: Desired LED state, an IndicatorLED value.
        :raises: InvalidParameterValueError, if any information passed is
            invalid.
        """
        try:
            state = res_cons.IndicatorLED(state).value
        except ValueError:
            raise exceptions.InvalidParameterValueError(
                parameter='state', value=state,
                valid_values=' ,'.join(i.value for i in res_cons.IndicatorLED))

        data = {'IndicatorLED': state}

        self._conn.patch(self.path, data=data)
        self.invalidate()

    @property
    @utils.cache_it
    def managers(self):
        """A list of managers for this chassis.

        Returns a list of `Manager` objects representing the managers
        that manage this chassis.

        :raises: MissingAttributeError if '@odata.id' field is missing.
        :returns: A list of `Manager` instances
        """
        paths = utils.get_sub_resource_path_by(
            self, ["Links", "ManagedBy"], is_collection=True)

        return [manager.Manager(self._conn, path,
                                redfish_version=self.redfish_version,
                                registries=self.registries,
                                root=self.root)
                for path in paths]

    @property
    @utils.cache_it
    def systems(self):
        """A list of systems residing in this chassis.

        Returns a list of `System` objects representing systems being
        mounted in this chassis/cabinet.

        :raises: MissingAttributeError if '@odata.id' field is missing.
        :returns: A list of `System` instances
        """
        paths = utils.get_sub_resource_path_by(
            self, ["Links", "ComputerSystems"], is_collection=True)

        from sushy.resources.system import system
        return [system.System(self._conn, path,
                              redfish_version=self.redfish_version,
                              registries=self.registries,
                              root=self.root)
                for path in paths]

    @property
    @utils.cache_it
    def power(self):
        """Property to reference `Power` instance

        It is set once when the first time it is queried. On refresh,
        this property is marked as stale (greedy-refresh not done).
        Here the actual refresh of the sub-resource happens, if stale.
        """
        return power.Power(
            self._conn,
            utils.get_sub_resource_path_by(self, 'Power'),
            redfish_version=self.redfish_version, registries=self.registries,
            root=self.root)

    @property
    @utils.cache_it
    def thermal(self):
        """Property to reference `Thermal` instance

        It is set once when the first time it is queried. On refresh,
        this property is marked as stale (greedy-refresh not done).
        Here the actual refresh of the sub-resource happens, if stale.
        """
        return thermal.Thermal(
            self._conn,
            utils.get_sub_resource_path_by(self, 'Thermal'),
            redfish_version=self.redfish_version, registries=self.registries,
            root=self.root)

    @property
    @utils.cache_it
    def network_adapters(self):
        """Property to reference `NetworkAdapterCollection` instance

        It is set once when the first time it is queried. On refresh,
        this property is marked as stale (greedy-refresh not done).
        Here the actual refresh of the sub-resource happens, if stale.
        """
        return adapter.NetworkAdapterCollection(
            self._conn,
            utils.get_sub_resource_path_by(self, "NetworkAdapters"),
            redfish_version=self.redfish_version,
            registries=self.registries, root=self.root)


class ChassisCollection(base.ResourceCollectionBase):

    @property
    def _resource_type(self):
        return Chassis

    def __init__(self, connector, path, redfish_version=None, registries=None,
                 root=None):
        """A class representing a ChassisCollection

        :param connector: A Connector instance
        :param path: The canonical path to the Chassis collection resource
        :param redfish_version: The version of RedFish. Used to construct
            the object according to schema of the given version.
        :param registries: Dict of Redfish Message Registry objects to be
            used in any resource that needs registries to parse messages
        :param root: Sushy root object. Empty for Sushy root itself.
        """
        super(ChassisCollection, self).__init__(
            connector, path, redfish_version=redfish_version,
            registries=registries, root=root)
